@@ -13,18 +13,24 @@ logger = logging.getLogger('django.contrib.gis')
 try:
     from django.conf import settings
     lib_path = settings.GDAL_LIBRARY_PATH
-except (AttributeError, EnvironmentError,
-        ImportError, ImproperlyConfigured):
+except (AttributeError, ImportError, ImproperlyConfigured, OSError):
     lib_path = None
 
 if lib_path:
     lib_names = None
 elif os.name == 'nt':
     # Windows NT shared libraries
-    lib_names = ['gdal203', 'gdal202', 'gdal201', 'gdal20', 'gdal111']
+    lib_names = [
+        'gdal303', 'gdal302', 'gdal301', 'gdal300',
+        'gdal204', 'gdal203', 'gdal202', 'gdal201', 'gdal20',
+    ]
 elif os.name == 'posix':
     # *NIX library names.
-    lib_names = ['gdal', 'GDAL', 'gdal2.3.0', 'gdal2.2.0', 'gdal2.1.0', 'gdal2.0.0', 'gdal1.11.0']
+    lib_names = [
+        'gdal', 'GDAL',
+        'gdal3.3.0', 'gdal3.2.0', 'gdal3.1.0', 'gdal3.0.0',
+        'gdal2.4.0', 'gdal2.3.0', 'gdal2.2.0', 'gdal2.1.0', 'gdal2.0.0',
+    ]
 else:
     raise ImproperlyConfigured('GDAL is unsupported on OS "%s".' % os.name)
 
@@ -81,26 +87,19 @@ def gdal_version():
 
 def gdal_full_version():
     "Return the full GDAL version information."
-    return _version_info('')
-
-
-version_regex = re.compile(r'^(?P<major>\d+)\.(?P<minor>\d+)(\.(?P<subminor>\d+))?')
+    return _version_info(b'')
 
 
 def gdal_version_info():
-    ver = gdal_version().decode()
-    m = version_regex.match(ver)
+    ver = gdal_version()
+    m = re.match(br'^(?P<major>\d+)\.(?P<minor>\d+)(?:\.(?P<subminor>\d+))?', ver)
     if not m:
         raise GDALException('Could not parse GDAL version string "%s"' % ver)
-    return {key: m.group(key) for key in ('major', 'minor', 'subminor')}
+    major, minor, subminor = m.groups()
+    return (int(major), int(minor), subminor and int(subminor))
 
 
-_verinfo = gdal_version_info()
-GDAL_MAJOR_VERSION = int(_verinfo['major'])
-GDAL_MINOR_VERSION = int(_verinfo['minor'])
-GDAL_SUBMINOR_VERSION = _verinfo['subminor'] and int(_verinfo['subminor'])
-GDAL_VERSION = (GDAL_MAJOR_VERSION, GDAL_MINOR_VERSION, GDAL_SUBMINOR_VERSION)
-del _verinfo
+GDAL_VERSION = gdal_version_info()
 
 # Set library error handling so as errors are logged
 CPLErrorHandler = CFUNCTYPE(None, c_int, c_int, c_char_p)
