@@ -1,3 +1,4 @@
+import _thread
 import copy
 import threading
 import time
@@ -5,8 +6,10 @@ import warnings
 from collections import deque
 from contextlib import contextmanager
 
-import _thread
-import pytz
+try:
+    import zoneinfo
+except ImportError:
+    from backports import zoneinfo
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
@@ -23,6 +26,14 @@ from django.utils.functional import cached_property
 NO_DB_ALIAS = '__no_db__'
 
 
+# RemovedInDjango50Warning
+def timezone_constructor(tzname):
+    if settings.USE_DEPRECATED_PYTZ:
+        import pytz
+        return pytz.timezone(tzname)
+    return zoneinfo.ZoneInfo(tzname)
+
+
 class BaseDatabaseWrapper:
     """Represent a database connection."""
     # Mapping of Field objects to their column types.
@@ -135,7 +146,7 @@ class BaseDatabaseWrapper:
         elif self.settings_dict['TIME_ZONE'] is None:
             return timezone.utc
         else:
-            return pytz.timezone(self.settings_dict['TIME_ZONE'])
+            return timezone_constructor(self.settings_dict['TIME_ZONE'])
 
     @cached_property
     def timezone_name(self):
@@ -632,6 +643,8 @@ class BaseDatabaseWrapper:
         return self.SchemaEditorClass(self, *args, **kwargs)
 
     def on_commit(self, func):
+        if not callable(func):
+            raise TypeError("on_commit()'s callback must be a callable.")
         if self.in_atomic_block:
             # Transaction in progress; save for execution on commit.
             self.run_on_commit.append((set(self.savepoint_ids), func))
