@@ -33,6 +33,7 @@ site.register(Event, EventAdmin)
 class ParentAdmin(admin.ModelAdmin):
     list_filter = ['child__name']
     search_fields = ['child__name']
+    list_select_related = ['child']
 
 
 class ChildAdmin(admin.ModelAdmin):
@@ -133,6 +134,7 @@ class NoListDisplayLinksParentAdmin(admin.ModelAdmin):
     list_display_links = None
     list_display = ['name']
     list_editable = ['name']
+    actions_on_bottom = True
 
 
 site.register(Parent, NoListDisplayLinksParentAdmin)
