@@ -2,6 +2,7 @@ import sys
 import unittest
 
 from django.conf import settings
+from django.contrib import admin
 from django.contrib.admindocs import utils, views
 from django.contrib.admindocs.views import get_return_data_type, simplify_regex
 from django.contrib.sites.models import Site
@@ -9,7 +10,8 @@ from django.db import models
 from django.db.models import fields
 from django.test import SimpleTestCase, modify_settings, override_settings
 from django.test.utils import captured_stderr
-from django.urls import reverse
+from django.urls import include, path, reverse
+from django.utils.functional import SimpleLazyObject
 
 from .models import Company, Person
 from .tests import AdminDocsTestCase, TestDataMixin
@@ -113,7 +115,7 @@ class AdminDocViewTests(TestDataMixin, AdminDocsTestCase):
             response = self.client.get(reverse('django-admindocs-docroot'))
             self.assertContains(
                 response,
-                '<h3>The admin documentation system requires Python\'s '
+                '<h3>The admin documentation system requires Python’s '
                 '<a href="https://docutils.sourceforge.io/">docutils</a> '
                 'library.</h3>'
                 '<p>Please ask your administrators to install '
@@ -136,6 +138,37 @@ class AdminDocViewTests(TestDataMixin, AdminDocsTestCase):
         response = self.client.get(reverse('django-admindocs-views-index'))
         self.assertContains(response, 'View documentation')
 
+    def test_callable_urlconf(self):
+        """
+        Index view should correctly resolve view patterns when ROOT_URLCONF is
+        not a string.
+        """
+        def urlpatterns():
+            return (
+                path('admin/doc/', include('django.contrib.admindocs.urls')),
+                path('admin/', admin.site.urls),
+            )
+
+        with self.settings(ROOT_URLCONF=SimpleLazyObject(urlpatterns)):
+            response = self.client.get(reverse('django-admindocs-views-index'))
+            self.assertEqual(response.status_code, 200)
+
+
+@unittest.skipUnless(utils.docutils_is_available, 'no docutils installed.')
+class AdminDocViewDefaultEngineOnly(TestDataMixin, AdminDocsTestCase):
+
+    def setUp(self):
+        self.client.force_login(self.superuser)
+
+    def test_template_detail_path_traversal(self):
+        cases = ['/etc/passwd', '../passwd']
+        for fpath in cases:
+            with self.subTest(path=fpath):
+                response = self.client.get(
+                    reverse('django-admindocs-templates', args=[fpath]),
+                )
+                self.assertEqual(response.status_code, 400)
+
 
 @override_settings(TEMPLATES=[{
     'NAME': 'ONE',
@@ -215,6 +248,10 @@ class TestModelDetailView(TestDataMixin, AdminDocsTestCase):
         """Model properties are displayed as fields."""
         self.assertContains(self.response, '<td>a_property</td>')
 
+    def test_instance_of_cached_property_methods_are_displayed(self):
+        """Model cached properties are displayed as fields."""
+        self.assertContains(self.response, '<td>a_cached_property</td>')
+
     def test_method_data_types(self):
         company = Company.objects.create(name="Django")
         person = Person.objects.create(first_name="Human", last_name="User", company=company)
