@@ -29,8 +29,12 @@ class Book(models.Model):
         blank=True, null=True,
     )
     is_best_seller = models.BooleanField(default=0, null=True)
-    is_best_seller2 = models.NullBooleanField(default=0)
     date_registered = models.DateField(null=True)
+    availability = models.BooleanField(choices=(
+        (False, 'Paid'),
+        (True, 'Free'),
+        (None, 'Obscure'),
+    ), null=True)
     # This field name is intentionally 2 characters long (#16080).
     no = models.IntegerField(verbose_name='number', blank=True, null=True)
 
