@@ -68,6 +68,15 @@ class ModelInstanceCreationTests(TestCase):
         a.save()
         self.assertEqual(a.headline, 'Fourth article')
 
+    def test_positional_and_keyword_args_for_the_same_field(self):
+        msg = "Article() got both positional and keyword arguments for field '%s'."
+        with self.assertRaisesMessage(TypeError, msg % 'headline'):
+            Article(None, 'Fifth article', headline='Other headline.')
+        with self.assertRaisesMessage(TypeError, msg % 'headline'):
+            Article(None, 'Sixth article', headline='')
+        with self.assertRaisesMessage(TypeError, msg % 'pub_date'):
+            Article(None, 'Seventh article', datetime(2021, 3, 1), pub_date=None)
+
     def test_cannot_create_instance_with_invalid_kwargs(self):
         with self.assertRaisesMessage(TypeError, "Article() got an unexpected keyword argument 'foo'"):
             Article(
@@ -161,13 +170,11 @@ class ModelTest(TestCase):
             Article(headline=headline, pub_date=some_pub_date).save()
         self.assertQuerysetEqual(
             Article.objects.all().order_by('headline'),
-            ["<Article: Amazing article>",
-             "<Article: An article>",
-             "<Article: Article One>",
-             "<Article: Boring article>"]
+            sorted(headlines),
+            transform=lambda a: a.headline,
         )
         Article.objects.filter(headline__startswith='A').delete()
-        self.assertQuerysetEqual(Article.objects.all().order_by('headline'), ["<Article: Boring article>"])
+        self.assertEqual(Article.objects.get().headline, 'Boring article')
 
     def test_not_equal_and_equal_operators_behave_as_expected_on_instances(self):
         some_pub_date = datetime(2014, 5, 16, 12, 1)
@@ -208,17 +215,17 @@ class ModelTest(TestCase):
     def test_year_lookup_edge_case(self):
         # Edge-case test: A year lookup should retrieve all objects in
         # the given year, including Jan. 1 and Dec. 31.
-        Article.objects.create(
+        a11 = Article.objects.create(
             headline='Article 11',
             pub_date=datetime(2008, 1, 1),
         )
-        Article.objects.create(
+        a12 = Article.objects.create(
             headline='Article 12',
             pub_date=datetime(2008, 12, 31, 23, 59, 59, 999999),
         )
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Article.objects.filter(pub_date__year=2008),
-            ["<Article: Article 11>", "<Article: Article 12>"]
+            [a11, a12],
         )
 
     def test_unicode_data(self):
@@ -442,7 +449,7 @@ class ModelLookupTest(TestCase):
         self.a.save()
 
         # Article.objects.all() returns all the articles in the database.
-        self.assertQuerysetEqual(Article.objects.all(), ['<Article: Parrot programs in Python>'])
+        self.assertSequenceEqual(Article.objects.all(), [self.a])
 
     def test_rich_lookup(self):
         # Django provides a rich database lookup API.
@@ -458,24 +465,24 @@ class ModelLookupTest(TestCase):
         self.assertEqual(Article.objects.get(id=self.a.id), self.a)
         self.assertEqual(Article.objects.get(headline='Swallow programs in Python'), self.a)
 
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Article.objects.filter(pub_date__year=2005),
-            ['<Article: Swallow programs in Python>'],
+            [self.a],
         )
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Article.objects.filter(pub_date__year=2004),
             [],
         )
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Article.objects.filter(pub_date__year=2005, pub_date__month=7),
-            ['<Article: Swallow programs in Python>'],
+            [self.a],
         )
 
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Article.objects.filter(pub_date__week_day=5),
-            ['<Article: Swallow programs in Python>'],
+            [self.a],
         )
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Article.objects.filter(pub_date__week_day=6),
             [],
         )
@@ -499,7 +506,7 @@ class ModelLookupTest(TestCase):
         self.assertEqual(Article.objects.get(pk=self.a.id), self.a)
 
         # pk can be used as a shortcut for the primary key name in any query.
-        self.assertQuerysetEqual(Article.objects.filter(pk__in=[self.a.id]), ["<Article: Swallow programs in Python>"])
+        self.assertSequenceEqual(Article.objects.filter(pk__in=[self.a.id]), [self.a])
 
         # Model instances of the same type and same ID are considered equal.
         a = Article.objects.get(pk=self.a.id)
@@ -574,6 +581,7 @@ class ManagerTest(SimpleTestCase):
         'filter',
         'aggregate',
         'annotate',
+        'alias',
         'complex_filter',
         'exclude',
         'in_bulk',
@@ -594,6 +602,7 @@ class ManagerTest(SimpleTestCase):
         'only',
         'using',
         'exists',
+        'contains',
         'explain',
         '_insert',
         '_update',
