@@ -425,6 +425,29 @@ class MethodDecoratorTests(SimpleTestCase):
                 def __module__(cls):
                     return "tests"
 
+    def test_wrapper_assignments(self):
+        """@method_decorator preserves wrapper assignments."""
+        func_name = None
+        func_module = None
+
+        def decorator(func):
+            @wraps(func)
+            def inner(*args, **kwargs):
+                nonlocal func_name, func_module
+                func_name = getattr(func, '__name__', None)
+                func_module = getattr(func, '__module__', None)
+                return func(*args, **kwargs)
+            return inner
+
+        class Test:
+            @method_decorator(decorator)
+            def method(self):
+                return 'tests'
+
+        Test().method()
+        self.assertEqual(func_name, 'method')
+        self.assertIsNotNone(func_module)
+
 
 class XFrameOptionsDecoratorsTests(TestCase):
     """
@@ -438,7 +461,7 @@ class XFrameOptionsDecoratorsTests(TestCase):
         def a_view(request):
             return HttpResponse()
         r = a_view(HttpRequest())
-        self.assertEqual(r['X-Frame-Options'], 'DENY')
+        self.assertEqual(r.headers['X-Frame-Options'], 'DENY')
 
     def test_sameorigin_decorator(self):
         """
@@ -449,7 +472,7 @@ class XFrameOptionsDecoratorsTests(TestCase):
         def a_view(request):
             return HttpResponse()
         r = a_view(HttpRequest())
-        self.assertEqual(r['X-Frame-Options'], 'SAMEORIGIN')
+        self.assertEqual(r.headers['X-Frame-Options'], 'SAMEORIGIN')
 
     def test_exempt_decorator(self):
         """
@@ -466,17 +489,81 @@ class XFrameOptionsDecoratorsTests(TestCase):
 
         # Since the real purpose of the exempt decorator is to suppress
         # the middleware's functionality, let's make sure it actually works...
-        r = XFrameOptionsMiddleware().process_response(req, resp)
+        r = XFrameOptionsMiddleware(a_view)(req)
         self.assertIsNone(r.get('X-Frame-Options', None))
 
 
-class NeverCacheDecoratorTest(TestCase):
+class HttpRequestProxy:
+    def __init__(self, request):
+        self._request = request
+
+    def __getattr__(self, attr):
+        """Proxy to the underlying HttpRequest object."""
+        return getattr(self._request, attr)
+
+
+class NeverCacheDecoratorTest(SimpleTestCase):
     def test_never_cache_decorator(self):
         @never_cache
         def a_view(request):
             return HttpResponse()
         r = a_view(HttpRequest())
         self.assertEqual(
-            set(r['Cache-Control'].split(', ')),
+            set(r.headers['Cache-Control'].split(', ')),
             {'max-age=0', 'no-cache', 'no-store', 'must-revalidate', 'private'},
         )
+
+    def test_never_cache_decorator_http_request(self):
+        class MyClass:
+            @never_cache
+            def a_view(self, request):
+                return HttpResponse()
+
+        request = HttpRequest()
+        msg = (
+            "never_cache didn't receive an HttpRequest. If you are decorating "
+            "a classmethod, be sure to use @method_decorator."
+        )
+        with self.assertRaisesMessage(TypeError, msg):
+            MyClass().a_view(request)
+        with self.assertRaisesMessage(TypeError, msg):
+            MyClass().a_view(HttpRequestProxy(request))
+
+    def test_never_cache_decorator_http_request_proxy(self):
+        class MyClass:
+            @method_decorator(never_cache)
+            def a_view(self, request):
+                return HttpResponse()
+
+        request = HttpRequest()
+        response = MyClass().a_view(HttpRequestProxy(request))
+        self.assertIn('Cache-Control', response.headers)
+        self.assertIn('Expires', response.headers)
+
+
+class CacheControlDecoratorTest(SimpleTestCase):
+    def test_cache_control_decorator_http_request(self):
+        class MyClass:
+            @cache_control(a='b')
+            def a_view(self, request):
+                return HttpResponse()
+
+        msg = (
+            "cache_control didn't receive an HttpRequest. If you are "
+            "decorating a classmethod, be sure to use @method_decorator."
+        )
+        request = HttpRequest()
+        with self.assertRaisesMessage(TypeError, msg):
+            MyClass().a_view(request)
+        with self.assertRaisesMessage(TypeError, msg):
+            MyClass().a_view(HttpRequestProxy(request))
+
+    def test_cache_control_decorator_http_request_proxy(self):
+        class MyClass:
+            @method_decorator(cache_control(a='b'))
+            def a_view(self, request):
+                return HttpResponse()
+
+        request = HttpRequest()
+        response = MyClass().a_view(HttpRequestProxy(request))
+        self.assertEqual(response.headers['Cache-Control'], 'a=b')
