@@ -1,11 +1,10 @@
-import warnings
-
+from django.apps import apps
 from django.db import models
-from django.test import TestCase, override_settings
-from django.utils import six
+from django.test import SimpleTestCase, override_settings
+from django.test.utils import isolate_lru_cache
 
 
-class FieldDeconstructionTests(TestCase):
+class FieldDeconstructionTests(SimpleTestCase):
     """
     Tests the deconstruct() method on all core fields.
     """
@@ -21,15 +20,35 @@ class FieldDeconstructionTests(TestCase):
         field.set_attributes_from_name("is_awesome_test")
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(name, "is_awesome_test")
-        self.assertIsInstance(name, six.text_type)
         # Now try with a ForeignKey
-        field = models.ForeignKey("some_fake.ModelName")
+        field = models.ForeignKey("some_fake.ModelName", models.CASCADE)
         name, path, args, kwargs = field.deconstruct()
         self.assertIsNone(name)
         field.set_attributes_from_name("author")
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(name, "author")
 
+    def test_db_tablespace(self):
+        field = models.Field()
+        _, _, args, kwargs = field.deconstruct()
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {})
+        # With a DEFAULT_DB_TABLESPACE.
+        with self.settings(DEFAULT_DB_TABLESPACE='foo'):
+            _, _, args, kwargs = field.deconstruct()
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {})
+        # With a db_tablespace.
+        field = models.Field(db_tablespace='foo')
+        _, _, args, kwargs = field.deconstruct()
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {'db_tablespace': 'foo'})
+        # With a db_tablespace equal to DEFAULT_DB_TABLESPACE.
+        with self.settings(DEFAULT_DB_TABLESPACE='foo'):
+            _, _, args, kwargs = field.deconstruct()
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {'db_tablespace': 'foo'})
+
     def test_auto_field(self):
         field = models.AutoField(primary_key=True)
         field.set_attributes_from_name("id")
@@ -69,6 +88,13 @@ class FieldDeconstructionTests(TestCase):
         self.assertEqual(args, [])
         self.assertEqual(kwargs, {"max_length": 65, "null": True, "blank": True})
 
+    def test_char_field_choices(self):
+        field = models.CharField(max_length=1, choices=(("A", "One"), ("B", "Two")))
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.CharField")
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {"choices": [("A", "One"), ("B", "Two")], "max_length": 1})
+
     def test_csi_field(self):
         field = models.CommaSeparatedIntegerField(max_length=100)
         name, path, args, kwargs = field.deconstruct()
@@ -128,7 +154,7 @@ class FieldDeconstructionTests(TestCase):
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(path, "django.db.models.EmailField")
         self.assertEqual(args, [])
-        self.assertEqual(kwargs, {"max_length": 75})
+        self.assertEqual(kwargs, {"max_length": 254})
         field = models.EmailField(max_length=255)
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(path, "django.db.models.EmailField")
@@ -149,11 +175,11 @@ class FieldDeconstructionTests(TestCase):
         self.assertEqual(kwargs, {"upload_to": "foo/bar", "max_length": 200})
 
     def test_file_path_field(self):
-        field = models.FilePathField(match=".*\.txt$")
+        field = models.FilePathField(match=r".*\.txt$")
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(path, "django.db.models.FilePathField")
         self.assertEqual(args, [])
-        self.assertEqual(kwargs, {"match": ".*\.txt$"})
+        self.assertEqual(kwargs, {"match": r".*\.txt$"})
         field = models.FilePathField(recursive=True, allow_folders=True, max_length=123)
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(path, "django.db.models.FilePathField")
@@ -170,57 +196,170 @@ class FieldDeconstructionTests(TestCase):
     def test_foreign_key(self):
         # Test basic pointing
         from django.contrib.auth.models import Permission
-        field = models.ForeignKey("auth.Permission")
-        field.rel.to = Permission
-        field.rel.field_name = "id"
+        field = models.ForeignKey("auth.Permission", models.CASCADE)
+        field.remote_field.model = Permission
+        field.remote_field.field_name = "id"
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(path, "django.db.models.ForeignKey")
         self.assertEqual(args, [])
-        self.assertEqual(kwargs, {"to": "auth.Permission"})
+        self.assertEqual(kwargs, {"to": "auth.permission", "on_delete": models.CASCADE})
         self.assertFalse(hasattr(kwargs['to'], "setting_name"))
         # Test swap detection for swappable model
-        field = models.ForeignKey("auth.User")
+        field = models.ForeignKey("auth.User", models.CASCADE)
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(path, "django.db.models.ForeignKey")
         self.assertEqual(args, [])
-        self.assertEqual(kwargs, {"to": "auth.User"})
+        self.assertEqual(kwargs, {"to": "auth.user", "on_delete": models.CASCADE})
         self.assertEqual(kwargs['to'].setting_name, "AUTH_USER_MODEL")
+        # Swap detection for lowercase swappable model.
+        field = models.ForeignKey('auth.user', models.CASCADE)
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, 'django.db.models.ForeignKey')
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {'to': 'auth.user', 'on_delete': models.CASCADE})
+        self.assertEqual(kwargs['to'].setting_name, 'AUTH_USER_MODEL')
         # Test nonexistent (for now) model
-        field = models.ForeignKey("something.Else")
+        field = models.ForeignKey("something.Else", models.CASCADE)
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(path, "django.db.models.ForeignKey")
         self.assertEqual(args, [])
-        self.assertEqual(kwargs, {"to": "something.Else"})
+        self.assertEqual(kwargs, {"to": "something.else", "on_delete": models.CASCADE})
         # Test on_delete
-        field = models.ForeignKey("auth.User", on_delete=models.SET_NULL)
+        field = models.ForeignKey("auth.User", models.SET_NULL)
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(path, "django.db.models.ForeignKey")
         self.assertEqual(args, [])
-        self.assertEqual(kwargs, {"to": "auth.User", "on_delete": models.SET_NULL})
+        self.assertEqual(kwargs, {"to": "auth.user", "on_delete": models.SET_NULL})
         # Test to_field preservation
-        field = models.ForeignKey("auth.Permission", to_field="foobar")
+        field = models.ForeignKey("auth.Permission", models.CASCADE, to_field="foobar")
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(path, "django.db.models.ForeignKey")
         self.assertEqual(args, [])
-        self.assertEqual(kwargs, {"to": "auth.Permission", "to_field": "foobar"})
+        self.assertEqual(kwargs, {"to": "auth.permission", "to_field": "foobar", "on_delete": models.CASCADE})
         # Test related_name preservation
-        field = models.ForeignKey("auth.Permission", related_name="foobar")
+        field = models.ForeignKey("auth.Permission", models.CASCADE, related_name="foobar")
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(path, "django.db.models.ForeignKey")
         self.assertEqual(args, [])
-        self.assertEqual(kwargs, {"to": "auth.Permission", "related_name": "foobar"})
+        self.assertEqual(kwargs, {"to": "auth.permission", "related_name": "foobar", "on_delete": models.CASCADE})
+        # Test related_query_name
+        field = models.ForeignKey("auth.Permission", models.CASCADE, related_query_name="foobar")
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.ForeignKey")
+        self.assertEqual(args, [])
+        self.assertEqual(
+            kwargs,
+            {"to": "auth.permission", "related_query_name": "foobar", "on_delete": models.CASCADE}
+        )
+        # Test limit_choices_to
+        field = models.ForeignKey("auth.Permission", models.CASCADE, limit_choices_to={'foo': 'bar'})
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.ForeignKey")
+        self.assertEqual(args, [])
+        self.assertEqual(
+            kwargs,
+            {"to": "auth.permission", "limit_choices_to": {'foo': 'bar'}, "on_delete": models.CASCADE}
+        )
+        # Test unique
+        field = models.ForeignKey("auth.Permission", models.CASCADE, unique=True)
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.ForeignKey")
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {"to": "auth.permission", "unique": True, "on_delete": models.CASCADE})
 
     @override_settings(AUTH_USER_MODEL="auth.Permission")
     def test_foreign_key_swapped(self):
-        # It doesn't matter that we swapped out user for permission;
-        # there's no validation. We just want to check the setting stuff works.
-        field = models.ForeignKey("auth.Permission")
-        name, path, args, kwargs = field.deconstruct()
+        with isolate_lru_cache(apps.get_swappable_settings_name):
+            # It doesn't matter that we swapped out user for permission;
+            # there's no validation. We just want to check the setting stuff works.
+            field = models.ForeignKey("auth.Permission", models.CASCADE)
+            name, path, args, kwargs = field.deconstruct()
+
         self.assertEqual(path, "django.db.models.ForeignKey")
         self.assertEqual(args, [])
-        self.assertEqual(kwargs, {"to": "auth.Permission"})
+        self.assertEqual(kwargs, {"to": "auth.permission", "on_delete": models.CASCADE})
         self.assertEqual(kwargs['to'].setting_name, "AUTH_USER_MODEL")
 
+        # Model names are case-insensitive.
+        with isolate_lru_cache(apps.get_swappable_settings_name):
+            # It doesn't matter that we swapped out user for permission;
+            # there's no validation. We just want to check the setting stuff
+            # works.
+            field = models.ForeignKey('auth.permission', models.CASCADE)
+            name, path, args, kwargs = field.deconstruct()
+
+        self.assertEqual(path, 'django.db.models.ForeignKey')
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {'to': 'auth.permission', 'on_delete': models.CASCADE})
+        self.assertEqual(kwargs['to'].setting_name, 'AUTH_USER_MODEL')
+
+    def test_one_to_one(self):
+        # Test basic pointing
+        from django.contrib.auth.models import Permission
+        field = models.OneToOneField("auth.Permission", models.CASCADE)
+        field.remote_field.model = Permission
+        field.remote_field.field_name = "id"
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.OneToOneField")
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {"to": "auth.permission", "on_delete": models.CASCADE})
+        self.assertFalse(hasattr(kwargs['to'], "setting_name"))
+        # Test swap detection for swappable model
+        field = models.OneToOneField("auth.User", models.CASCADE)
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.OneToOneField")
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {"to": "auth.user", "on_delete": models.CASCADE})
+        self.assertEqual(kwargs['to'].setting_name, "AUTH_USER_MODEL")
+        # Test nonexistent (for now) model
+        field = models.OneToOneField("something.Else", models.CASCADE)
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.OneToOneField")
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {"to": "something.else", "on_delete": models.CASCADE})
+        # Test on_delete
+        field = models.OneToOneField("auth.User", models.SET_NULL)
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.OneToOneField")
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {"to": "auth.user", "on_delete": models.SET_NULL})
+        # Test to_field
+        field = models.OneToOneField("auth.Permission", models.CASCADE, to_field="foobar")
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.OneToOneField")
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {"to": "auth.permission", "to_field": "foobar", "on_delete": models.CASCADE})
+        # Test related_name
+        field = models.OneToOneField("auth.Permission", models.CASCADE, related_name="foobar")
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.OneToOneField")
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {"to": "auth.permission", "related_name": "foobar", "on_delete": models.CASCADE})
+        # Test related_query_name
+        field = models.OneToOneField("auth.Permission", models.CASCADE, related_query_name="foobar")
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.OneToOneField")
+        self.assertEqual(args, [])
+        self.assertEqual(
+            kwargs,
+            {"to": "auth.permission", "related_query_name": "foobar", "on_delete": models.CASCADE}
+        )
+        # Test limit_choices_to
+        field = models.OneToOneField("auth.Permission", models.CASCADE, limit_choices_to={'foo': 'bar'})
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.OneToOneField")
+        self.assertEqual(args, [])
+        self.assertEqual(
+            kwargs,
+            {"to": "auth.permission", "limit_choices_to": {'foo': 'bar'}, "on_delete": models.CASCADE}
+        )
+        # Test unique
+        field = models.OneToOneField("auth.Permission", models.CASCADE, unique=True)
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.OneToOneField")
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {"to": "auth.permission", "on_delete": models.CASCADE})
+
     def test_image_field(self):
         field = models.ImageField(upload_to="foo/barness", width_field="width", height_field="height")
         name, path, args, kwargs = field.deconstruct()
@@ -236,9 +375,7 @@ class FieldDeconstructionTests(TestCase):
         self.assertEqual(kwargs, {})
 
     def test_ip_address_field(self):
-        with warnings.catch_warnings(record=True):
-            warnings.simplefilter("always")
-            field = models.IPAddressField()
+        field = models.IPAddressField()
         name, path, args, kwargs = field.deconstruct()
         self.assertEqual(path, "django.db.models.IPAddressField")
         self.assertEqual(args, [])
@@ -289,24 +426,57 @@ class FieldDeconstructionTests(TestCase):
         self.assertEqual(path, "django.db.models.ManyToManyField")
         self.assertEqual(args, [])
         self.assertEqual(kwargs, {"to": "auth.Permission", "related_name": "custom_table"})
+        # Test related_query_name
+        field = models.ManyToManyField("auth.Permission", related_query_name="foobar")
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.ManyToManyField")
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {"to": "auth.Permission", "related_query_name": "foobar"})
+        # Test limit_choices_to
+        field = models.ManyToManyField("auth.Permission", limit_choices_to={'foo': 'bar'})
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, "django.db.models.ManyToManyField")
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {"to": "auth.Permission", "limit_choices_to": {'foo': 'bar'}})
 
     @override_settings(AUTH_USER_MODEL="auth.Permission")
     def test_many_to_many_field_swapped(self):
-        # It doesn't matter that we swapped out user for permission;
-        # there's no validation. We just want to check the setting stuff works.
-        field = models.ManyToManyField("auth.Permission")
-        name, path, args, kwargs = field.deconstruct()
+        with isolate_lru_cache(apps.get_swappable_settings_name):
+            # It doesn't matter that we swapped out user for permission;
+            # there's no validation. We just want to check the setting stuff works.
+            field = models.ManyToManyField("auth.Permission")
+            name, path, args, kwargs = field.deconstruct()
+
         self.assertEqual(path, "django.db.models.ManyToManyField")
         self.assertEqual(args, [])
         self.assertEqual(kwargs, {"to": "auth.Permission"})
         self.assertEqual(kwargs['to'].setting_name, "AUTH_USER_MODEL")
 
-    def test_null_boolean_field(self):
-        field = models.NullBooleanField()
-        name, path, args, kwargs = field.deconstruct()
-        self.assertEqual(path, "django.db.models.NullBooleanField")
+    def test_many_to_many_field_related_name(self):
+        class MyModel(models.Model):
+            flag = models.BooleanField(default=True)
+            m2m = models.ManyToManyField('self')
+            m2m_related_name = models.ManyToManyField(
+                'self',
+                related_query_name='custom_query_name',
+                limit_choices_to={'flag': True},
+            )
+
+        name, path, args, kwargs = MyModel.m2m.field.deconstruct()
+        self.assertEqual(path, 'django.db.models.ManyToManyField')
         self.assertEqual(args, [])
-        self.assertEqual(kwargs, {})
+        # deconstruct() should not include attributes which were not passed to
+        # the field during initialization.
+        self.assertEqual(kwargs, {'to': 'field_deconstruction.MyModel'})
+        # Passed attributes.
+        name, path, args, kwargs = MyModel.m2m_related_name.field.deconstruct()
+        self.assertEqual(path, 'django.db.models.ManyToManyField')
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {
+            'to': 'field_deconstruction.MyModel',
+            'related_query_name': 'custom_query_name',
+            'limit_choices_to': {'flag': True},
+        })
 
     def test_positive_integer_field(self):
         field = models.PositiveIntegerField()
@@ -322,6 +492,13 @@ class FieldDeconstructionTests(TestCase):
         self.assertEqual(args, [])
         self.assertEqual(kwargs, {})
 
+    def test_positive_big_integer_field(self):
+        field = models.PositiveBigIntegerField()
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(path, 'django.db.models.PositiveBigIntegerField')
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {})
+
     def test_slug_field(self):
         field = models.SlugField()
         name, path, args, kwargs = field.deconstruct()
@@ -383,3 +560,7 @@ class FieldDeconstructionTests(TestCase):
         self.assertEqual(path, "django.db.models.BinaryField")
         self.assertEqual(args, [])
         self.assertEqual(kwargs, {})
+        field = models.BinaryField(editable=True)
+        name, path, args, kwargs = field.deconstruct()
+        self.assertEqual(args, [])
+        self.assertEqual(kwargs, {'editable': True})
