@@ -28,6 +28,7 @@ from django.test import (
 from django.test.utils import requires_tz_support
 from django.urls import NoReverseMatch, reverse_lazy
 from django.utils import timezone
+from django.utils._os import symlinks_supported
 
 from .models import (
     Storage, callable_storage, temp_storage, temp_storage_location,
@@ -297,6 +298,22 @@ class FileStorageTests(SimpleTestCase):
 
         self.storage.delete('path/to/test.file')
 
+    def test_file_save_abs_path(self):
+        test_name = 'path/to/test.file'
+        f = ContentFile('file saved with path')
+        f_name = self.storage.save(os.path.join(self.temp_dir, test_name), f)
+        self.assertEqual(f_name, test_name)
+
+    @unittest.skipUnless(symlinks_supported(), 'Must be able to symlink to run this test.')
+    def test_file_save_broken_symlink(self):
+        """A new path is created on save when a broken symlink is supplied."""
+        nonexistent_file_path = os.path.join(self.temp_dir, 'nonexistent.txt')
+        broken_symlink_path = os.path.join(self.temp_dir, 'symlink.txt')
+        os.symlink(nonexistent_file_path, broken_symlink_path)
+        f = ContentFile('some content')
+        f_name = self.storage.save(broken_symlink_path, f)
+        self.assertIs(os.path.exists(os.path.join(self.temp_dir, f_name)), True)
+
     def test_save_doesnt_close(self):
         with TemporaryUploadedFile('test', 'text/plain', 1, 'utf8') as file:
             file.write(b'1')
@@ -328,7 +345,7 @@ class FileStorageTests(SimpleTestCase):
 
     def test_file_url(self):
         """
-        File storage returns a url to access a given file from the Web.
+        File storage returns a url to access a given file from the web.
         """
         self.assertEqual(self.storage.url('test.file'), self.storage.base_url + 'test.file')
 
@@ -501,7 +518,10 @@ class FileStorageTests(SimpleTestCase):
         Calling delete with an empty name should not try to remove the base
         storage directory, but fail loudly (#20660).
         """
-        with self.assertRaises(AssertionError):
+        msg = 'The name must be given to delete().'
+        with self.assertRaisesMessage(ValueError, msg):
+            self.storage.delete(None)
+        with self.assertRaisesMessage(ValueError, msg):
             self.storage.delete('')
 
     def test_delete_deletes_directories(self):
