@@ -1,5 +1,4 @@
 import tempfile
-import unittest
 from io import StringIO
 
 from django.contrib.gis import gdal
@@ -12,11 +11,9 @@ from django.core.management import call_command
 from django.db import DatabaseError, NotSupportedError, connection
 from django.db.models import F, OuterRef, Subquery
 from django.test import TestCase, skipUnlessDBFeature
+from django.test.utils import CaptureQueriesContext
 
-from ..utils import (
-    mariadb, mysql, no_oracle, oracle, postgis, skipUnlessGISLookup,
-    spatialite,
-)
+from ..utils import skipUnlessGISLookup
 from .models import (
     City, Country, Feature, MinusOneSRID, MultiFields, NonConcreteModel,
     PennsylvaniaCity, State, Track,
@@ -79,7 +76,7 @@ class GeoModelTest(TestCase):
         nullstate.save()
 
         ns = State.objects.get(name='NullState')
-        self.assertEqual(ply, ns.poly)
+        self.assertEqual(connection.ops.Adapter._fix_polygon(ply), ns.poly)
 
         # Testing the `ogr` and `srs` lazy-geometry properties.
         self.assertIsInstance(ns.poly.ogr, gdal.OGRGeometry)
@@ -93,7 +90,10 @@ class GeoModelTest(TestCase):
         ply[1] = new_inner
         self.assertEqual(4326, ns.poly.srid)
         ns.save()
-        self.assertEqual(ply, State.objects.get(name='NullState').poly)
+        self.assertEqual(
+            connection.ops.Adapter._fix_polygon(ply),
+            State.objects.get(name='NullState').poly
+        )
         ns.delete()
 
     @skipUnlessDBFeature("supports_transform")
@@ -107,7 +107,7 @@ class GeoModelTest(TestCase):
         # Constructing & querying with a point from a different SRID. Oracle
         # `SDO_OVERLAPBDYINTERSECT` operates differently from
         # `ST_Intersects`, so contains is used instead.
-        if oracle:
+        if connection.ops.oracle:
             tx = Country.objects.get(mpoly__contains=other_srid_pnt)
         else:
             tx = Country.objects.get(mpoly__intersects=other_srid_pnt)
@@ -154,9 +154,6 @@ class GeoModelTest(TestCase):
         self.assertIsInstance(f_4.geom, GeometryCollection)
         self.assertEqual(f_3.geom, f_4.geom[2])
 
-    # TODO: fix on Oracle: ORA-22901: cannot compare nested table or VARRAY or
-    # LOB attributes of an object type.
-    @no_oracle
     @skipUnlessDBFeature("supports_transform")
     def test_inherited_geofields(self):
         "Database functions on inherited Geometry fields."
@@ -229,8 +226,6 @@ class GeoLookupTest(TestCase):
 
     def test_disjoint_lookup(self):
         "Testing the `disjoint` lookup type."
-        if mysql and not mariadb and connection.mysql_version < (8, 0, 0):
-            raise unittest.SkipTest('MySQL < 8 gives different results.')
         ptown = City.objects.get(name='Pueblo')
         qs1 = City.objects.filter(point__disjoint=ptown.point)
         self.assertEqual(7, qs1.count())
@@ -302,7 +297,7 @@ class GeoLookupTest(TestCase):
         invalid_geom = fromstr('POLYGON((0 0, 0 1, 1 1, 1 0, 1 1, 1 0, 0 0))')
         State.objects.create(name='invalid', poly=invalid_geom)
         qs = State.objects.all()
-        if oracle or (mysql and connection.mysql_version < (8, 0, 0)):
+        if connection.ops.oracle or (connection.ops.mysql and connection.mysql_version < (8, 0, 0)):
             # Kansas has adjacent vertices with distance 6.99244813842e-12
             # which is smaller than the default Oracle tolerance.
             # It's invalid on MySQL < 8 also.
@@ -311,7 +306,7 @@ class GeoLookupTest(TestCase):
         self.assertEqual(qs.filter(poly__isvalid=False).count(), 1)
         self.assertEqual(qs.filter(poly__isvalid=True).count(), qs.count() - 1)
 
-    @skipUnlessDBFeature("supports_left_right_lookups")
+    @skipUnlessGISLookup('left', 'right')
     def test_left_right_lookups(self):
         "Testing the 'left' and 'right' lookup types."
         # Left: A << B => true if xmax(A) < xmin(B)
@@ -432,9 +427,8 @@ class GeoLookupTest(TestCase):
 
     def test_wkt_string_in_lookup(self):
         # Valid WKT strings don't emit error logs.
-        with self.assertRaisesMessage(AssertionError, 'no logs'):
-            with self.assertLogs('django.contrib.gis', 'ERROR'):
-                State.objects.filter(poly__intersects='LINESTRING(0 0, 1 1, 5 5)')
+        with self.assertNoLogs('django.contrib.gis', 'ERROR'):
+            State.objects.filter(poly__intersects='LINESTRING(0 0, 1 1, 5 5)')
 
     @skipUnlessDBFeature("supports_relate_lookup")
     def test_relate_lookup(self):
@@ -456,12 +450,11 @@ class GeoLookupTest(TestCase):
             with self.assertRaises(e):
                 qs.count()
 
-        # Relate works differently for the different backends.
-        if postgis or spatialite or mariadb:
-            contains_mask = 'T*T***FF*'
-            within_mask = 'T*F**F***'
-            intersects_mask = 'T********'
-        elif oracle:
+        contains_mask = 'T*T***FF*'
+        within_mask = 'T*F**F***'
+        intersects_mask = 'T********'
+        # Relate works differently on Oracle.
+        if connection.ops.oracle:
             contains_mask = 'contains'
             within_mask = 'inside'
             # TODO: This is not quite the same as the PostGIS mask above
@@ -480,7 +473,7 @@ class GeoLookupTest(TestCase):
         self.assertEqual('Lawrence', City.objects.get(point__relate=(ks.poly, within_mask)).name)
 
         # Testing intersection relation mask.
-        if not oracle:
+        if not connection.ops.oracle:
             if connection.features.supports_transform:
                 self.assertEqual(
                     Country.objects.get(mpoly__relate=(pnt1, intersects_mask)).name,
@@ -490,7 +483,7 @@ class GeoLookupTest(TestCase):
             self.assertEqual('Lawrence', City.objects.get(point__relate=(ks.poly, intersects_mask)).name)
 
         # With a complex geometry expression
-        mask = 'anyinteract' if oracle else within_mask
+        mask = 'anyinteract' if connection.ops.oracle else within_mask
         self.assertFalse(City.objects.exclude(point__relate=(functions.Union('point', 'point'), mask)))
 
     def test_gis_lookups_with_complex_expressions(self):
@@ -594,10 +587,20 @@ class GeoQuerySetTest(TestCase):
         qs = City.objects.filter(name='NotACity')
         self.assertIsNone(qs.aggregate(Union('point'))['point__union'])
 
-    @unittest.skipUnless(
-        connection.vendor == 'oracle',
-        'Oracle supports tolerance parameter.',
-    )
+    @skipUnlessDBFeature('supports_union_aggr')
+    def test_geoagg_subquery(self):
+        tx = Country.objects.get(name='Texas')
+        union = GEOSGeometry('MULTIPOINT(-96.801611 32.782057,-95.363151 29.763374)')
+        # Use distinct() to force the usage of a subquery for aggregation.
+        with CaptureQueriesContext(connection) as ctx:
+            self.assertIs(union.equals(
+                City.objects.filter(point__within=tx.mpoly).distinct().aggregate(
+                    Union('point'),
+                )['point__union'],
+            ), True)
+        self.assertIn('subquery', ctx.captured_queries[0]['sql'])
+
+    @skipUnlessDBFeature('supports_tolerance_parameter')
     def test_unionagg_tolerance(self):
         City.objects.create(
             point=fromstr('POINT(-96.467222 32.751389)', srid=4326),
@@ -620,10 +623,7 @@ class GeoQuerySetTest(TestCase):
             True,
         )
 
-    @unittest.skipUnless(
-        connection.vendor == 'oracle',
-        'Oracle supports tolerance parameter.',
-    )
+    @skipUnlessDBFeature('supports_tolerance_parameter')
     def test_unionagg_tolerance_escaping(self):
         tx = Country.objects.get(name='Texas').mpoly
         with self.assertRaises(DatabaseError):
