@@ -8,7 +8,7 @@ from django.contrib.gis.geos import GEOSGeometry
 from django.contrib.gis.measure import D
 from django.contrib.gis.shortcuts import numpy
 from django.db import connection
-from django.db.models import Q
+from django.db.models import F, Func, Q
 from django.test import TransactionTestCase, skipUnlessDBFeature
 from django.test.utils import CaptureQueriesContext
 
@@ -51,6 +51,29 @@ class RasterFieldTest(TransactionTestCase):
         qs = RasterModel.objects.all()
         qs[0].rast.bands[0].data()
 
+    def test_deserialize_with_pixeltype_flags(self):
+        no_data = 3
+        rast = GDALRaster({
+            'srid': 4326,
+            'origin': [0, 0],
+            'scale': [-1, 1],
+            'skew': [0, 0],
+            'width': 1,
+            'height': 1,
+            'nr_of_bands': 1,
+            'bands': [{'data': [no_data], 'nodata_value': no_data}],
+        })
+        r = RasterModel.objects.create(rast=rast)
+        RasterModel.objects.filter(pk=r.pk).update(
+            rast=Func(F('rast'), function='ST_SetBandIsNoData'),
+        )
+        r.refresh_from_db()
+        band = r.rast.bands[0].data()
+        if numpy:
+            band = band.flatten().tolist()
+        self.assertEqual(band, [no_data])
+        self.assertEqual(r.rast.bands[0].nodata_value, no_data)
+
     def test_model_creation(self):
         """
         Test RasterField through a test model.
@@ -120,7 +143,9 @@ class RasterFieldTest(TransactionTestCase):
         unprojected coordinate systems. This test just checks that the lookup
         can be called, but doesn't check if the result makes logical sense.
         """
-        from django.contrib.gis.db.backends.postgis.operations import PostGISOperations
+        from django.contrib.gis.db.backends.postgis.operations import (
+            PostGISOperations,
+        )
 
         # Create test raster and geom.
         rast = GDALRaster(json.loads(JSON_RASTER))
@@ -304,12 +329,12 @@ class RasterFieldTest(TransactionTestCase):
 
     def test_lookup_with_raster_bbox(self):
         rast = GDALRaster(json.loads(JSON_RASTER))
-        # Shift raster upwards
+        # Shift raster upward
         rast.origin.y = 2
         # The raster in the model is not strictly below
         qs = RasterModel.objects.filter(rast__strictly_below=rast)
         self.assertEqual(qs.count(), 0)
-        # Shift raster further upwards
+        # Shift raster further upward
         rast.origin.y = 6
         # The raster in the model is strictly below
         qs = RasterModel.objects.filter(rast__strictly_below=rast)
