@@ -1,23 +1,22 @@
-from unittest import skipUnless
+import re
 
 from django.contrib.gis import forms
-from django.contrib.gis.gdal import HAS_GDAL
+from django.contrib.gis.forms import BaseGeometryWidget, OpenLayersWidget
 from django.contrib.gis.geos import GEOSGeometry
-from django.forms import ValidationError
-from django.test import SimpleTestCase, override_settings, skipUnlessDBFeature
-from django.utils import six
+from django.core.exceptions import ValidationError
+from django.test import SimpleTestCase, override_settings
 from django.utils.html import escape
 
 
-@skipUnless(HAS_GDAL, "GeometryFieldTest needs GDAL support")
-@skipUnlessDBFeature("gis_enabled")
 class GeometryFieldTest(SimpleTestCase):
 
     def test_init(self):
         "Testing GeometryField initialization with defaults."
         fld = forms.GeometryField()
         for bad_default in ('blah', 3, 'FoO', None, 0):
-            self.assertRaises(ValidationError, fld.clean, bad_default)
+            with self.subTest(bad_default=bad_default):
+                with self.assertRaises(ValidationError):
+                    fld.clean(bad_default)
 
     def test_srid(self):
         "Testing GeometryField with a SRID set."
@@ -29,18 +28,20 @@ class GeometryFieldTest(SimpleTestCase):
         # Making the field in a different SRID from that of the geometry, and
         # asserting it transforms.
         fld = forms.GeometryField(srid=32140)
-        tol = 0.0000001
+        # Different PROJ versions use different transformations, all are
+        # correct as having a 1 meter accuracy.
+        tol = 1
         xform_geom = GEOSGeometry('POINT (951640.547328465 4219369.26171664)', srid=32140)
-        # The cleaned geometry should be transformed to 32140.
-        cleaned_geom = fld.clean('SRID=4326;POINT (-95.363151 29.763374)')
+        # The cleaned geometry is transformed to 32140 (the widget map_srid is 3857).
+        cleaned_geom = fld.clean('SRID=3857;POINT (-10615777.40976205 3473169.895707852)')
+        self.assertEqual(cleaned_geom.srid, 32140)
         self.assertTrue(xform_geom.equals_exact(cleaned_geom, tol))
 
     def test_null(self):
         "Testing GeometryField's handling of null (None) geometries."
         # Form fields, by default, are required (`required=True`)
         fld = forms.GeometryField()
-        with six.assertRaisesRegex(self, forms.ValidationError,
-                "No geometry value provided."):
+        with self.assertRaisesMessage(ValidationError, "No geometry value provided."):
             fld.clean(None)
 
         # This will clean None as a geometry (See #10660).
@@ -52,27 +53,54 @@ class GeometryFieldTest(SimpleTestCase):
         # By default, all geometry types are allowed.
         fld = forms.GeometryField()
         for wkt in ('POINT(5 23)', 'MULTIPOLYGON(((0 0, 0 1, 1 1, 1 0, 0 0)))', 'LINESTRING(0 0, 1 1)'):
-            self.assertEqual(GEOSGeometry(wkt), fld.clean(wkt))
+            with self.subTest(wkt=wkt):
+                # to_python() uses the SRID of OpenLayersWidget if the
+                # converted value doesn't have an SRID.
+                self.assertEqual(GEOSGeometry(wkt, srid=fld.widget.map_srid), fld.clean(wkt))
 
         pnt_fld = forms.GeometryField(geom_type='POINT')
-        self.assertEqual(GEOSGeometry('POINT(5 23)'), pnt_fld.clean('POINT(5 23)'))
+        self.assertEqual(GEOSGeometry('POINT(5 23)', srid=pnt_fld.widget.map_srid), pnt_fld.clean('POINT(5 23)'))
         # a WKT for any other geom_type will be properly transformed by `to_python`
-        self.assertEqual(GEOSGeometry('LINESTRING(0 0, 1 1)'), pnt_fld.to_python('LINESTRING(0 0, 1 1)'))
+        self.assertEqual(
+            GEOSGeometry('LINESTRING(0 0, 1 1)', srid=pnt_fld.widget.map_srid),
+            pnt_fld.to_python('LINESTRING(0 0, 1 1)')
+        )
         # but rejected by `clean`
-        self.assertRaises(forms.ValidationError, pnt_fld.clean, 'LINESTRING(0 0, 1 1)')
+        with self.assertRaises(ValidationError):
+            pnt_fld.clean('LINESTRING(0 0, 1 1)')
 
     def test_to_python(self):
         """
-        Testing to_python returns a correct GEOSGeometry object or
-        a ValidationError
+        to_python() either returns a correct GEOSGeometry object or
+        a ValidationError.
         """
+        good_inputs = [
+            'POINT(5 23)',
+            'MULTIPOLYGON(((0 0, 0 1, 1 1, 1 0, 0 0)))',
+            'LINESTRING(0 0, 1 1)',
+        ]
+        bad_inputs = [
+            'POINT(5)',
+            'MULTI   POLYGON(((0 0, 0 1, 1 1, 1 0, 0 0)))',
+            'BLAH(0 0, 1 1)',
+            '{"type": "FeatureCollection", "features": ['
+            '{"geometry": {"type": "Point", "coordinates": [508375, 148905]}, "type": "Feature"}]}',
+        ]
         fld = forms.GeometryField()
         # to_python returns the same GEOSGeometry for a WKT
-        for wkt in ('POINT(5 23)', 'MULTIPOLYGON(((0 0, 0 1, 1 1, 1 0, 0 0)))', 'LINESTRING(0 0, 1 1)'):
-            self.assertEqual(GEOSGeometry(wkt), fld.to_python(wkt))
+        for geo_input in good_inputs:
+            with self.subTest(geo_input=geo_input):
+                self.assertEqual(GEOSGeometry(geo_input, srid=fld.widget.map_srid), fld.to_python(geo_input))
         # but raises a ValidationError for any other string
-        for wkt in ('POINT(5)', 'MULTI   POLYGON(((0 0, 0 1, 1 1, 1 0, 0 0)))', 'BLAH(0 0, 1 1)'):
-            self.assertRaises(forms.ValidationError, fld.to_python, wkt)
+        for geo_input in bad_inputs:
+            with self.subTest(geo_input=geo_input):
+                with self.assertRaises(ValidationError):
+                    fld.to_python(geo_input)
+
+    def test_to_python_different_map_srid(self):
+        f = forms.GeometryField(widget=OpenLayersWidget)
+        json = '{ "type": "Point", "coordinates": [ 5.0, 23.0 ] }'
+        self.assertEqual(GEOSGeometry('POINT(5 23)', srid=f.widget.map_srid), f.to_python(json))
 
     def test_field_with_text_widget(self):
         class PointForm(forms.Form):
@@ -80,16 +108,61 @@ class GeometryFieldTest(SimpleTestCase):
 
         form = PointForm()
         cleaned_pt = form.fields['pt'].clean('POINT(5 23)')
-        self.assertEqual(cleaned_pt, GEOSGeometry('POINT(5 23)'))
+        self.assertEqual(cleaned_pt, GEOSGeometry('POINT(5 23)', srid=4326))
         self.assertEqual(4326, cleaned_pt.srid)
+        with self.assertRaisesMessage(ValidationError, 'Invalid geometry value.'):
+            form.fields['pt'].clean('POINT(5)')
 
         point = GEOSGeometry('SRID=4326;POINT(5 23)')
         form = PointForm(data={'pt': 'POINT(5 23)'}, initial={'pt': point})
         self.assertFalse(form.has_changed())
 
+    def test_field_string_value(self):
+        """
+        Initialization of a geometry field with a valid/empty/invalid string.
+        Only the invalid string should trigger an error log entry.
+        """
+        class PointForm(forms.Form):
+            pt1 = forms.PointField(srid=4326)
+            pt2 = forms.PointField(srid=4326)
+            pt3 = forms.PointField(srid=4326)
+
+        form = PointForm({
+            'pt1': 'SRID=4326;POINT(7.3 44)',  # valid
+            'pt2': '',  # empty
+            'pt3': 'PNT(0)',  # invalid
+        })
+
+        with self.assertLogs('django.contrib.gis', 'ERROR') as logger_calls:
+            output = str(form)
+
+        # The first point can't use assertInHTML() due to non-deterministic
+        # ordering of the rendered dictionary.
+        pt1_serialized = re.search(r'<textarea [^>]*>({[^<]+})<', output)[1]
+        pt1_json = pt1_serialized.replace('&quot;', '"')
+        pt1_expected = GEOSGeometry(form.data['pt1']).transform(3857, clone=True)
+        self.assertJSONEqual(pt1_json, pt1_expected.json)
+
+        self.assertInHTML(
+            '<textarea id="id_pt2" class="vSerializedField required" cols="150"'
+            ' rows="10" name="pt2"></textarea>',
+            output
+        )
+        self.assertInHTML(
+            '<textarea id="id_pt3" class="vSerializedField required" cols="150"'
+            ' rows="10" name="pt3"></textarea>',
+            output
+        )
+        # Only the invalid PNT(0) triggers an error log entry.
+        # Deserialization is called in form clean and in widget rendering.
+        self.assertEqual(len(logger_calls.records), 2)
+        self.assertEqual(
+            logger_calls.records[0].getMessage(),
+            "Error creating geometry from value 'PNT(0)' (String input "
+            "unrecognized as WKT EWKT, and HEXEWKB.)"
+        )
+
 
-@skipUnless(HAS_GDAL, "SpecializedFieldTest needs GDAL support")
-@skipUnlessDBFeature("gis_enabled")
 class SpecializedFieldTest(SimpleTestCase):
     def setUp(self):
         self.geometries = {
@@ -152,7 +225,7 @@ class SpecializedFieldTest(SimpleTestCase):
         self.assertTrue(form_instance.is_valid())
         rendered = form_instance.as_p()
         self.assertIn('new MapWidget(options);', rendered)
-        self.assertIn('map_srid: 4326,', rendered)
+        self.assertIn('map_srid: 3857,', rendered)
         self.assertIn('gis/js/OLMapWidget.js', str(form_instance.media))
 
     def assertTextarea(self, geom, rendered):
@@ -160,10 +233,12 @@ class SpecializedFieldTest(SimpleTestCase):
 
         self.assertIn('<textarea ', rendered)
         self.assertIn('required', rendered)
-        self.assertIn(geom.wkt, rendered)
+        ogr = geom.ogr
+        ogr.transform(3857)
+        self.assertIn(escape(ogr.json), rendered)
 
-    # map_srid in operlayers.html template must not be localized.
-    @override_settings(USE_L10N=True, USE_THOUSAND_SEPARATOR=True)
+    # map_srid in openlayers.html template must not be localized.
+    @override_settings(USE_THOUSAND_SEPARATOR=True)
     def test_pointfield(self):
         class PointForm(forms.Form):
             p = forms.PointField()
@@ -195,10 +270,10 @@ class SpecializedFieldTest(SimpleTestCase):
 
     def test_linestringfield(self):
         class LineStringForm(forms.Form):
-            l = forms.LineStringField()
+            f = forms.LineStringField()
 
         geom = self.geometries['linestring']
-        form = LineStringForm(data={'l': geom})
+        form = LineStringForm(data={'f': geom})
         self.assertTextarea(geom, form.as_p())
         self.assertMapWidget(form)
         self.assertFalse(LineStringForm().is_valid())
@@ -208,10 +283,10 @@ class SpecializedFieldTest(SimpleTestCase):
 
     def test_multilinestringfield(self):
         class LineStringForm(forms.Form):
-            l = forms.MultiLineStringField()
+            f = forms.MultiLineStringField()
 
         geom = self.geometries['multilinestring']
-        form = LineStringForm(data={'l': geom})
+        form = LineStringForm(data={'f': geom})
         self.assertTextarea(geom, form.as_p())
         self.assertMapWidget(form)
         self.assertFalse(LineStringForm().is_valid())
@@ -259,8 +334,6 @@ class SpecializedFieldTest(SimpleTestCase):
             self.assertFalse(GeometryForm(data={'g': invalid.wkt}).is_valid())
 
 
-@skipUnless(HAS_GDAL, "OSMWidgetTest needs GDAL support")
-@skipUnlessDBFeature("gis_enabled")
 class OSMWidgetTest(SimpleTestCase):
     def setUp(self):
         self.geometries = {
@@ -275,14 +348,20 @@ class OSMWidgetTest(SimpleTestCase):
         form = PointForm(data={'p': geom})
         rendered = form.as_p()
 
-        self.assertIn("OpenStreetMap (Mapnik)", rendered)
+        self.assertIn("ol.source.OSM()", rendered)
         self.assertIn("id: 'id_p',", rendered)
 
     def test_default_lat_lon(self):
+        self.assertEqual(forms.OSMWidget.default_lon, 5)
+        self.assertEqual(forms.OSMWidget.default_lat, 47)
+        self.assertEqual(forms.OSMWidget.default_zoom, 12)
+
         class PointForm(forms.Form):
             p = forms.PointField(
                 widget=forms.OSMWidget(attrs={
-                    'default_lon': 20, 'default_lat': 30
+                    'default_lon': 20,
+                    'default_lat': 30,
+                    'default_zoom': 17,
                 }),
             )
 
@@ -291,19 +370,44 @@ class OSMWidgetTest(SimpleTestCase):
 
         self.assertIn("options['default_lon'] = 20;", rendered)
         self.assertIn("options['default_lat'] = 30;", rendered)
-        if forms.OSMWidget.default_lon != 20:
-            self.assertNotIn(
-                "options['default_lon'] = %d;" % forms.OSMWidget.default_lon,
-                rendered)
-        if forms.OSMWidget.default_lat != 30:
-            self.assertNotIn(
-                "options['default_lat'] = %d;" % forms.OSMWidget.default_lat,
-                rendered)
+        self.assertIn("options['default_zoom'] = 17;", rendered)
 
 
-@skipUnless(HAS_GDAL, "CustomGeometryWidgetTest needs GDAL support")
-@skipUnlessDBFeature("gis_enabled")
-class CustomGeometryWidgetTest(SimpleTestCase):
+class GeometryWidgetTests(SimpleTestCase):
+
+    def test_get_context_attrs(self):
+        # The Widget.get_context() attrs argument overrides self.attrs.
+        widget = BaseGeometryWidget(attrs={'geom_type': 'POINT'})
+        context = widget.get_context('point', None, attrs={'geom_type': 'POINT2'})
+        self.assertEqual(context['geom_type'], 'POINT2')
+        # Widget.get_context() returns expected name for geom_type.
+        widget = BaseGeometryWidget(attrs={'geom_type': 'POLYGON'})
+        context = widget.get_context('polygon', None, None)
+        self.assertEqual(context['geom_type'], 'Polygon')
+        # Widget.get_context() returns 'Geometry' instead of 'Unknown'.
+        widget = BaseGeometryWidget(attrs={'geom_type': 'GEOMETRY'})
+        context = widget.get_context('geometry', None, None)
+        self.assertEqual(context['geom_type'], 'Geometry')
+
+    def test_subwidgets(self):
+        widget = forms.BaseGeometryWidget()
+        self.assertEqual(
+            list(widget.subwidgets('name', 'value')),
+            [{
+                'is_hidden': False,
+                'attrs': {
+                    'map_srid': 4326,
+                    'map_width': 600,
+                    'geom_type': 'GEOMETRY',
+                    'map_height': 400,
+                    'display_raw': False,
+                },
+                'name': 'name',
+                'template_name': '',
+                'value': 'value',
+                'required': False,
+            }]
+        )
 
     def test_custom_serialization_widget(self):
         class CustomGeometryWidget(forms.BaseGeometryWidget):
@@ -332,5 +436,4 @@ class CustomGeometryWidgetTest(SimpleTestCase):
 
         form = PointForm(data={'p': point.json})
         self.assertTrue(form.is_valid())
-        # Ensure that resulting geometry has srid set
         self.assertEqual(form.cleaned_data['p'].srid, 4326)
