@@ -1,8 +1,8 @@
 import datetime
 
 from django import forms
-from django.core.validators import ValidationError
-from django.forms.models import ModelChoiceIterator
+from django.core.exceptions import ValidationError
+from django.forms.models import ModelChoiceIterator, ModelChoiceIteratorValue
 from django.forms.widgets import CheckboxSelectMultiple
 from django.template import Context, Template
 from django.test import TestCase
@@ -139,6 +139,26 @@ class ModelChoiceFieldTests(TestCase):
         Category.objects.all().delete()
         self.assertIs(bool(f.choices), True)
 
+    def test_choices_radio_blank(self):
+        choices = [
+            (self.c1.pk, 'Entertainment'),
+            (self.c2.pk, 'A test'),
+            (self.c3.pk, 'Third'),
+        ]
+        categories = Category.objects.all()
+        for widget in [forms.RadioSelect, forms.RadioSelect()]:
+            for blank in [True, False]:
+                with self.subTest(widget=widget, blank=blank):
+                    f = forms.ModelChoiceField(
+                        categories,
+                        widget=widget,
+                        blank=blank,
+                    )
+                    self.assertEqual(
+                        list(f.choices),
+                        [('', '---------')] + choices if blank else choices,
+                    )
+
     def test_deepcopies_widget(self):
         class ModelChoiceForm(forms.Form):
             category = forms.ModelChoiceField(Category.objects.all())
@@ -260,6 +280,32 @@ class ModelChoiceFieldTests(TestCase):
         self.assertIsInstance(field.choices, CustomModelChoiceIterator)
 
     def test_choice_iterator_passes_model_to_widget(self):
+        class CustomCheckboxSelectMultiple(CheckboxSelectMultiple):
+            def create_option(self, name, value, label, selected, index, subindex=None, attrs=None):
+                option = super().create_option(name, value, label, selected, index, subindex, attrs)
+                # Modify the HTML based on the object being rendered.
+                c = value.instance
+                option['attrs']['data-slug'] = c.slug
+                return option
+
+        class CustomModelMultipleChoiceField(forms.ModelMultipleChoiceField):
+            widget = CustomCheckboxSelectMultiple
+
+        field = CustomModelMultipleChoiceField(Category.objects.all())
+        self.assertHTMLEqual(
+            field.widget.render('name', []), (
+                '<div>'
+                '<div><label><input type="checkbox" name="name" value="%d" '
+                'data-slug="entertainment">Entertainment</label></div>'
+                '<div><label><input type="checkbox" name="name" value="%d" '
+                'data-slug="test">A test</label></div>'
+                '<div><label><input type="checkbox" name="name" value="%d" '
+                'data-slug="third-test">Third</label></div>'
+                '</div>'
+            ) % (self.c1.pk, self.c2.pk, self.c3.pk),
+        )
+
+    def test_custom_choice_iterator_passes_model_to_widget(self):
         class CustomModelChoiceValue:
             def __init__(self, value, obj):
                 self.value = value
@@ -275,7 +321,7 @@ class ModelChoiceFieldTests(TestCase):
 
         class CustomCheckboxSelectMultiple(CheckboxSelectMultiple):
             def create_option(self, name, value, label, selected, index, subindex=None, attrs=None):
-                option = super().create_option(name, value, label, selected, index, subindex=None, attrs=None)
+                option = super().create_option(name, value, label, selected, index, subindex, attrs)
                 # Modify the HTML based on the object being rendered.
                 c = value.obj
                 option['attrs']['data-slug'] = c.slug
@@ -288,13 +334,19 @@ class ModelChoiceFieldTests(TestCase):
         field = CustomModelMultipleChoiceField(Category.objects.all())
         self.assertHTMLEqual(
             field.widget.render('name', []),
-            '''<ul>
-<li><label><input type="checkbox" name="name" value="%d" data-slug="entertainment">Entertainment</label></li>
-<li><label><input type="checkbox" name="name" value="%d" data-slug="test">A test</label></li>
-<li><label><input type="checkbox" name="name" value="%d" data-slug="third-test">Third</label></li>
-</ul>''' % (self.c1.pk, self.c2.pk, self.c3.pk),
+            """<div>
+<div><label><input type="checkbox" name="name" value="%d" data-slug="entertainment">Entertainment</label></div>
+<div><label><input type="checkbox" name="name" value="%d" data-slug="test">A test</label></div>
+<div><label><input type="checkbox" name="name" value="%d" data-slug="third-test">Third</label></div>
+</div>""" % (self.c1.pk, self.c2.pk, self.c3.pk),
         )
 
+    def test_choice_value_hash(self):
+        value_1 = ModelChoiceIteratorValue(self.c1.pk, self.c1)
+        value_2 = ModelChoiceIteratorValue(self.c2.pk, self.c2)
+        self.assertEqual(hash(value_1), hash(ModelChoiceIteratorValue(self.c1.pk, None)))
+        self.assertNotEqual(hash(value_1), hash(value_2))
+
     def test_choices_not_fetched_when_not_rendering(self):
         with self.assertNumQueries(1):
             field = forms.ModelChoiceField(Category.objects.order_by('-name'))
