@@ -70,8 +70,8 @@ class GenericIPAddressData(models.Model):
     data = models.GenericIPAddressField(null=True)
 
 
-class NullBooleanData(models.Model):
-    data = models.NullBooleanField(null=True)
+class PositiveBigIntegerData(models.Model):
+    data = models.PositiveBigIntegerField(null=True)
 
 
 class PositiveIntegerData(models.Model):
