@@ -28,13 +28,31 @@ from django.core import mail
 from django.http import HttpResponse, HttpResponseNotAllowed
 from django.test import (
     AsyncRequestFactory, Client, RequestFactory, SimpleTestCase, TestCase,
-    override_settings,
+    modify_settings, override_settings,
 )
 from django.urls import reverse_lazy
+from django.utils.decorators import async_only_middleware
 
 from .views import TwoArgException, get_view, post_view, trace_view
 
 
+def middleware_urlconf(get_response):
+    def middleware(request):
+        request.urlconf = 'tests.test_client.urls_middleware_urlconf'
+        return get_response(request)
+
+    return middleware
+
+
+@async_only_middleware
+def async_middleware_urlconf(get_response):
+    async def middleware(request):
+        request.urlconf = 'tests.test_client.urls_middleware_urlconf'
+        return await get_response(request)
+
+    return middleware
+
+
 @override_settings(ROOT_URLCONF='test_client.urls')
 class ClientTest(TestCase):
 
@@ -96,10 +114,9 @@ class ClientTest(TestCase):
         response = self.client.post('/post_view/', post_data)
 
         # Check some response details
-        self.assertEqual(response.status_code, 200)
+        self.assertContains(response, 'Data received')
         self.assertEqual(response.context['data'], '37')
         self.assertEqual(response.templates[0].name, 'POST Template')
-        self.assertContains(response, 'Data received')
 
     def test_post_data_none(self):
         msg = (
@@ -124,9 +141,8 @@ class ClientTest(TestCase):
                         client_method = getattr(self.client, method)
                         method_name = method.upper()
                         response = client_method('/json_view/', data, content_type='application/json')
-                        self.assertEqual(response.status_code, 200)
-                        self.assertEqual(response.context['data'], expected)
                         self.assertContains(response, 'Viewing %s page.' % method_name)
+                        self.assertEqual(response.context['data'], expected)
 
     def test_json_encoder_argument(self):
         """The test Client accepts a json_encoder."""
@@ -159,7 +175,7 @@ class ClientTest(TestCase):
         "Check the value of HTTP headers returned in a response"
         response = self.client.get("/header_view/")
 
-        self.assertEqual(response['X-DJANGO-TEST'], 'Slartibartfast')
+        self.assertEqual(response.headers['X-DJANGO-TEST'], 'Slartibartfast')
 
     def test_response_attached_request(self):
         """
@@ -197,6 +213,11 @@ class ClientTest(TestCase):
         response = self.client.get('/get_view/')
         self.assertEqual(response.resolver_match.url_name, 'get_view')
 
+    @modify_settings(MIDDLEWARE={'prepend': 'test_client.tests.middleware_urlconf'})
+    def test_response_resolver_match_middleware_urlconf(self):
+        response = self.client.get('/middleware_urlconf_view/')
+        self.assertEqual(response.resolver_match.url_name, 'middleware_urlconf_view')
+
     def test_raw_post(self):
         "POST raw data (with a content type) to a view"
         test_doc = """<?xml version="1.0" encoding="utf-8"?>
@@ -270,6 +291,13 @@ class ClientTest(TestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.request['PATH_INFO'], '/accounts/login/')
 
+    def test_redirect_to_querystring_only(self):
+        """A URL that consists of a querystring only can be followed"""
+        response = self.client.post('/post_then_get_view/', follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.request['PATH_INFO'], '/post_then_get_view/')
+        self.assertEqual(response.content, b'The value of success is true.')
+
     def test_follow_307_and_308_redirect(self):
         """
         A 307 or 308 redirect preserves the request method after the redirect.
@@ -284,6 +312,34 @@ class ClientTest(TestCase):
                 self.assertEqual(response.request['PATH_INFO'], '/post_view/')
                 self.assertEqual(response.request['REQUEST_METHOD'], method.upper())
 
+    def test_follow_307_and_308_preserves_query_string(self):
+        methods = ('post', 'options', 'put', 'patch', 'delete', 'trace')
+        codes = (307, 308)
+        for method, code in itertools.product(methods, codes):
+            with self.subTest(method=method, code=code):
+                req_method = getattr(self.client, method)
+                response = req_method(
+                    '/redirect_view_%s_query_string/' % code,
+                    data={'value': 'test'},
+                    follow=True,
+                )
+                self.assertRedirects(response, '/post_view/?hello=world', status_code=code)
+                self.assertEqual(response.request['QUERY_STRING'], 'hello=world')
+
+    def test_follow_307_and_308_get_head_query_string(self):
+        methods = ('get', 'head')
+        codes = (307, 308)
+        for method, code in itertools.product(methods, codes):
+            with self.subTest(method=method, code=code):
+                req_method = getattr(self.client, method)
+                response = req_method(
+                    '/redirect_view_%s_query_string/' % code,
+                    data={'value': 'test'},
+                    follow=True,
+                )
+                self.assertRedirects(response, '/post_view/?hello=world', status_code=code)
+                self.assertEqual(response.request['QUERY_STRING'], 'value=test')
+
     def test_follow_307_and_308_preserves_post_data(self):
         for code in (307, 308):
             with self.subTest(code=code):
@@ -304,12 +360,12 @@ class ClientTest(TestCase):
                 self.assertContains(response, '30 is the value')
 
     def test_redirect_http(self):
-        "GET a URL that redirects to an http URI"
+        """GET a URL that redirects to an HTTP URI."""
         response = self.client.get('/http_redirect_view/', follow=True)
         self.assertFalse(response.test_was_secure_request)
 
     def test_redirect_https(self):
-        "GET a URL that redirects to an https URI"
+        """GET a URL that redirects to an HTTPS URI."""
         response = self.client.get('/https_redirect_view/', follow=True)
         self.assertTrue(response.test_was_secure_request)
 
@@ -338,10 +394,9 @@ class ClientTest(TestCase):
             'multi': ('b', 'c', 'e')
         }
         response = self.client.get('/form_view/', data=hints)
-        self.assertEqual(response.status_code, 200)
-        self.assertTemplateUsed(response, "Form GET Template")
         # The multi-value data has been rolled out ok
         self.assertContains(response, 'Select a valid choice.', 0)
+        self.assertTemplateUsed(response, "Form GET Template")
 
     def test_incomplete_data_form(self):
         "POST incomplete data to a form"
@@ -351,7 +406,6 @@ class ClientTest(TestCase):
         }
         response = self.client.post('/form_view/', post_data)
         self.assertContains(response, 'This field is required.', 3)
-        self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(response, "Invalid POST Template")
 
         self.assertFormError(response, 'form', 'email', 'This field is required.')
@@ -715,6 +769,13 @@ class ClientTest(TestCase):
         response = self.client.get('/django_project_redirect/')
         self.assertRedirects(response, 'https://www.djangoproject.com/', fetch_redirect_response=False)
 
+    def test_external_redirect_without_trailing_slash(self):
+        """
+        Client._handle_redirects() with an empty path.
+        """
+        response = self.client.get('/no_trailing_slash_external_redirect/', follow=True)
+        self.assertRedirects(response, 'https://testserver')
+
     def test_external_redirect_with_fetch_error_msg(self):
         """
         assertRedirects without fetch_redirect_response=False raises
@@ -840,8 +901,11 @@ class ClientTest(TestCase):
         self.assertEqual(response.content, b'temp_file')
 
     def test_uploading_named_temp_file(self):
-        test_file = tempfile.NamedTemporaryFile()
-        response = self.client.post('/upload_view/', data={'named_temp_file': test_file})
+        with tempfile.NamedTemporaryFile() as test_file:
+            response = self.client.post(
+                '/upload_view/',
+                data={'named_temp_file': test_file},
+            )
         self.assertEqual(response.content, b'named_temp_file')
 
 
@@ -928,6 +992,13 @@ class AsyncClientTest(TestCase):
         self.assertTrue(hasattr(response, 'resolver_match'))
         self.assertEqual(response.resolver_match.url_name, 'async_get_view')
 
+    @modify_settings(
+        MIDDLEWARE={'prepend': 'test_client.tests.async_middleware_urlconf'},
+    )
+    async def test_response_resolver_match_middleware_urlconf(self):
+        response = await self.async_client.get('/middleware_urlconf_view/')
+        self.assertEqual(response.resolver_match.url_name, 'middleware_urlconf_view')
+
     async def test_follow_parameter_not_implemented(self):
         msg = 'AsyncClient request methods do not accept the follow parameter.'
         tests = (
@@ -946,6 +1017,10 @@ class AsyncClientTest(TestCase):
                 with self.assertRaisesMessage(NotImplementedError, msg):
                     await method('/redirect_view/', follow=True)
 
+    async def test_get_data(self):
+        response = await self.async_client.get('/get_view/', {'var': 'val'})
+        self.assertContains(response, 'This is a test. val is the value.')
+
 
 @override_settings(ROOT_URLCONF='test_client.urls')
 class AsyncRequestFactoryTest(SimpleTestCase):
@@ -999,3 +1074,8 @@ class AsyncRequestFactoryTest(SimpleTestCase):
         self.assertIn('HTTP_AUTHORIZATION', request.META)
         self.assertEqual(request.headers['x-another-header'], 'some other value')
         self.assertIn('HTTP_X_ANOTHER_HEADER', request.META)
+
+    def test_request_factory_query_string(self):
+        request = self.request_factory.get('/somewhere/', {'example': 'data'})
+        self.assertNotIn('Query-String', request.headers)
+        self.assertEqual(request.GET['example'], 'data')
