@@ -5,7 +5,15 @@ from contextlib import contextmanager
 from unittest import SkipTest, skipIf
 from xml.dom.minidom import parseString
 
-import pytz
+try:
+    import zoneinfo
+except ImportError:
+    from backports import zoneinfo
+
+try:
+    import pytz
+except ImportError:
+    pytz = None
 
 from django.contrib.auth.models import User
 from django.core import serializers
@@ -16,12 +24,13 @@ from django.template import (
     Context, RequestContext, Template, TemplateSyntaxError, context_processors,
 )
 from django.test import (
-    SimpleTestCase, TestCase, TransactionTestCase, override_settings,
-    skipIfDBFeature, skipUnlessDBFeature,
+    SimpleTestCase, TestCase, TransactionTestCase, ignore_warnings,
+    override_settings, skipIfDBFeature, skipUnlessDBFeature,
 )
 from django.test.utils import requires_tz_support
 from django.urls import reverse
 from django.utils import timezone
+from django.utils.deprecation import RemovedInDjango50Warning
 from django.utils.timezone import timedelta
 
 from .forms import (
@@ -29,7 +38,8 @@ from .forms import (
     EventSplitForm,
 )
 from .models import (
-    AllDayEvent, Event, MaybeEvent, Session, SessionEvent, Timestamp,
+    AllDayEvent, DailyEvent, Event, MaybeEvent, Session, SessionEvent,
+    Timestamp,
 )
 
 try:
@@ -39,7 +49,7 @@ except ImportError:
     HAS_YAML = False
 
 # These tests use the EAT (Eastern Africa Time) and ICT (Indochina Time)
-# who don't have Daylight Saving Time, so we can represent them easily
+# who don't have daylight saving time, so we can represent them easily
 # with fixed offset timezones and use them directly as tzinfo in the
 # constructors.
 
@@ -51,6 +61,14 @@ UTC = timezone.utc
 EAT = timezone.get_fixed_timezone(180)      # Africa/Nairobi
 ICT = timezone.get_fixed_timezone(420)      # Asia/Bangkok
 
+ZONE_CONSTRUCTORS = (zoneinfo.ZoneInfo,)
+if pytz is not None:
+    ZONE_CONSTRUCTORS += (pytz.timezone,)
+
+
+def get_timezones(key):
+    return [constructor(key) for constructor in ZONE_CONSTRUCTORS]
+
 
 @contextmanager
 def override_database_connection_timezone(timezone):
@@ -254,6 +272,13 @@ class LegacyDatabaseTests(TestCase):
 class NewDatabaseTests(TestCase):
     naive_warning = 'DateTimeField Event.dt received a naive datetime'
 
+    @skipIfDBFeature('supports_timezones')
+    def test_aware_time_unsupported(self):
+        t = datetime.time(13, 20, 30, tzinfo=EAT)
+        msg = 'backend does not support timezone-aware times.'
+        with self.assertRaisesMessage(ValueError, msg):
+            DailyEvent.objects.create(time=t)
+
     @requires_tz_support
     def test_naive_datetime(self):
         dt = datetime.datetime(2011, 9, 1, 13, 20, 30)
@@ -326,16 +351,34 @@ class NewDatabaseTests(TestCase):
         self.assertEqual(Event.objects.filter(dt__gt=dt2).count(), 0)
 
     def test_query_filter_with_pytz_timezones(self):
-        tz = pytz.timezone('Europe/Paris')
-        dt = datetime.datetime(2011, 9, 1, 12, 20, 30, tzinfo=tz)
-        Event.objects.create(dt=dt)
-        next = dt + datetime.timedelta(seconds=3)
-        prev = dt - datetime.timedelta(seconds=3)
-        self.assertEqual(Event.objects.filter(dt__exact=dt).count(), 1)
-        self.assertEqual(Event.objects.filter(dt__exact=next).count(), 0)
-        self.assertEqual(Event.objects.filter(dt__in=(prev, next)).count(), 0)
-        self.assertEqual(Event.objects.filter(dt__in=(prev, dt, next)).count(), 1)
-        self.assertEqual(Event.objects.filter(dt__range=(prev, next)).count(), 1)
+        for tz in get_timezones('Europe/Paris'):
+            with self.subTest(repr(tz)):
+                dt = datetime.datetime(2011, 9, 1, 12, 20, 30, tzinfo=tz)
+                Event.objects.create(dt=dt)
+                next = dt + datetime.timedelta(seconds=3)
+                prev = dt - datetime.timedelta(seconds=3)
+                self.assertEqual(Event.objects.filter(dt__exact=dt).count(), 1)
+                self.assertEqual(Event.objects.filter(dt__exact=next).count(), 0)
+                self.assertEqual(Event.objects.filter(dt__in=(prev, next)).count(), 0)
+                self.assertEqual(Event.objects.filter(dt__in=(prev, dt, next)).count(), 1)
+                self.assertEqual(Event.objects.filter(dt__range=(prev, next)).count(), 1)
+
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_connection_timezone(self):
+        tests = [
+            (False, None, datetime.timezone),
+            (False, 'Africa/Nairobi', zoneinfo.ZoneInfo),
+        ]
+        if pytz is not None:
+            tests += [
+                (True, None, datetime.timezone),
+                (True, 'Africa/Nairobi', pytz.BaseTzInfo),
+            ]
+        for use_pytz, connection_tz, expected_type in tests:
+            with self.subTest(use_pytz=use_pytz, connection_tz=connection_tz):
+                with self.settings(USE_DEPRECATED_PYTZ=use_pytz):
+                    with override_database_connection_timezone(connection_tz):
+                        self.assertIsInstance(connection.timezone, expected_type)
 
     def test_query_convert_timezones(self):
         # Connection timezone is equal to the current timezone, datetime
@@ -543,7 +586,7 @@ class NewDatabaseTests(TestCase):
             with connection.cursor() as cursor:
                 cursor.execute('SELECT CURRENT_TIMESTAMP')
                 now = cursor.fetchone()[0]
-                self.assertEqual(now.tzinfo.zone, 'Europe/Paris')
+                self.assertEqual(str(now.tzinfo), 'Europe/Paris')
 
     @requires_tz_support
     def test_filter_date_field_with_aware_datetime(self):
@@ -787,8 +830,15 @@ class SerializationTests(SimpleTestCase):
                 self.assertEqual(obj.dt, dt)
 
 
+# RemovedInDjango50Warning: When the deprecation ends, remove setUpClass() and
+# USE_L10N=False. The tests should remain because format-related settings will
+# take precedence over locale-dictated formats.
 @override_settings(DATETIME_FORMAT='c', TIME_ZONE='Africa/Nairobi', USE_L10N=False, USE_TZ=True)
 class TemplateTests(SimpleTestCase):
+    @classmethod
+    def setUpClass(cls):
+        with ignore_warnings(category=RemovedInDjango50Warning):
+            super().setUpClass()
 
     @requires_tz_support
     def test_localtime_templatetag_and_filters(self):
@@ -871,32 +921,26 @@ class TemplateTests(SimpleTestCase):
                     expected = results[k1][k2]
                     self.assertEqual(actual, expected, '%s / %s: %r != %r' % (k1, k2, actual, expected))
 
-    def test_localtime_filters_with_pytz(self):
+    def test_localtime_filters_with_iana(self):
         """
-        Test the |localtime, |utc, and |timezone filters with pytz.
+        Test the |localtime, |utc, and |timezone filters with iana zones.
         """
-        # Use a pytz timezone as local time
+        # Use an IANA timezone as local time
         tpl = Template("{% load tz %}{{ dt|localtime }}|{{ dt|utc }}")
         ctx = Context({'dt': datetime.datetime(2011, 9, 1, 12, 20, 30)})
 
         with self.settings(TIME_ZONE='Europe/Paris'):
             self.assertEqual(tpl.render(ctx), "2011-09-01T12:20:30+02:00|2011-09-01T10:20:30+00:00")
 
-        # Use a pytz timezone as argument
-        tpl = Template("{% load tz %}{{ dt|timezone:tz }}")
-        ctx = Context({
-            'dt': datetime.datetime(2011, 9, 1, 13, 20, 30),
-            'tz': pytz.timezone('Europe/Paris'),
-        })
-        self.assertEqual(tpl.render(ctx), "2011-09-01T12:20:30+02:00")
-
-        # Use a pytz timezone name as argument
-        tpl = Template("{% load tz %}{{ dt|timezone:'Europe/Paris' }}")
-        ctx = Context({
-            'dt': datetime.datetime(2011, 9, 1, 13, 20, 30),
-            'tz': pytz.timezone('Europe/Paris'),
-        })
-        self.assertEqual(tpl.render(ctx), "2011-09-01T12:20:30+02:00")
+        # Use an IANA timezone as argument
+        for tz in get_timezones('Europe/Paris'):
+            with self.subTest(repr(tz)):
+                tpl = Template("{% load tz %}{{ dt|timezone:tz }}")
+                ctx = Context({
+                    'dt': datetime.datetime(2011, 9, 1, 13, 20, 30),
+                    'tz': tz,
+                })
+                self.assertEqual(tpl.render(ctx), "2011-09-01T12:20:30+02:00")
 
     def test_localtime_templatetag_invalid_argument(self):
         with self.assertRaises(TemplateSyntaxError):
@@ -945,31 +989,37 @@ class TemplateTests(SimpleTestCase):
             "2011-09-01T13:20:30+03:00|2011-09-01T17:20:30+07:00|2011-09-01T13:20:30+03:00"
         )
 
-    def test_timezone_templatetag_with_pytz(self):
+    def test_timezone_templatetag_with_iana(self):
         """
-        Test the {% timezone %} templatetag with pytz.
+        Test the {% timezone %} templatetag with IANA time zone providers.
         """
         tpl = Template("{% load tz %}{% timezone tz %}{{ dt }}{% endtimezone %}")
 
-        # Use a pytz timezone as argument
-        ctx = Context({
-            'dt': datetime.datetime(2011, 9, 1, 13, 20, 30, tzinfo=EAT),
-            'tz': pytz.timezone('Europe/Paris'),
-        })
-        self.assertEqual(tpl.render(ctx), "2011-09-01T12:20:30+02:00")
+        # Use a IANA timezone as argument
+        for tz in get_timezones('Europe/Paris'):
+            with self.subTest(repr(tz)):
+                ctx = Context({
+                    'dt': datetime.datetime(2011, 9, 1, 13, 20, 30, tzinfo=EAT),
+                    'tz': tz,
+                })
+                self.assertEqual(tpl.render(ctx), "2011-09-01T12:20:30+02:00")
 
-        # Use a pytz timezone name as argument
+        # Use a IANA timezone name as argument
         ctx = Context({
             'dt': datetime.datetime(2011, 9, 1, 13, 20, 30, tzinfo=EAT),
             'tz': 'Europe/Paris',
         })
         self.assertEqual(tpl.render(ctx), "2011-09-01T12:20:30+02:00")
 
+    @ignore_warnings(category=RemovedInDjango50Warning)
     def test_timezone_templatetag_invalid_argument(self):
         with self.assertRaises(TemplateSyntaxError):
             Template("{% load tz %}{% timezone %}{% endtimezone %}").render()
-        with self.assertRaises(pytz.UnknownTimeZoneError):
+        with self.assertRaises(zoneinfo.ZoneInfoNotFoundError):
             Template("{% load tz %}{% timezone tz %}{% endtimezone %}").render(Context({'tz': 'foobar'}))
+        if pytz is not None:
+            with override_settings(USE_DEPRECATED_PYTZ=True), self.assertRaises(pytz.UnknownTimeZoneError):
+                Template("{% load tz %}{% timezone tz %}{% endtimezone %}").render(Context({'tz': 'foobar'}))
 
     @skipIf(sys.platform == 'win32', "Windows uses non-standard time zone names")
     def test_get_current_timezone_templatetag(self):
@@ -991,13 +1041,15 @@ class TemplateTests(SimpleTestCase):
         with timezone.override(UTC):
             self.assertEqual(tpl.render(Context({'tz': ICT})), "+0700")
 
-    def test_get_current_timezone_templatetag_with_pytz(self):
+    def test_get_current_timezone_templatetag_with_iana(self):
         """
         Test the {% get_current_timezone %} templatetag with pytz.
         """
         tpl = Template("{% load tz %}{% get_current_timezone as time_zone %}{{ time_zone }}")
-        with timezone.override(pytz.timezone('Europe/Paris')):
-            self.assertEqual(tpl.render(Context()), "Europe/Paris")
+        for tz in get_timezones('Europe/Paris'):
+            with self.subTest(repr(tz)):
+                with timezone.override(tz):
+                    self.assertEqual(tpl.render(Context()), "Europe/Paris")
 
         tpl = Template(
             "{% load tz %}{% timezone 'Europe/Paris' %}"
@@ -1049,8 +1101,15 @@ class TemplateTests(SimpleTestCase):
             self.assertEqual(tpl.render(Context({})), "+0700")
 
 
+# RemovedInDjango50Warning: When the deprecation ends, remove setUpClass() and
+# USE_L10N=False. The tests should remain because format-related settings will
+# take precedence over locale-dictated formats.
 @override_settings(DATETIME_FORMAT='c', TIME_ZONE='Africa/Nairobi', USE_L10N=False, USE_TZ=False)
 class LegacyFormsTests(TestCase):
+    @classmethod
+    def setUpClass(cls):
+        with ignore_warnings(category=RemovedInDjango50Warning):
+            super().setUpClass()
 
     def test_form(self):
         form = EventForm({'dt': '2011-09-01 13:20:30'})
@@ -1059,17 +1118,21 @@ class LegacyFormsTests(TestCase):
 
     def test_form_with_non_existent_time(self):
         form = EventForm({'dt': '2011-03-27 02:30:00'})
-        with timezone.override(pytz.timezone('Europe/Paris')):
-            # This is a bug.
-            self.assertTrue(form.is_valid())
-            self.assertEqual(form.cleaned_data['dt'], datetime.datetime(2011, 3, 27, 2, 30, 0))
+        for tz in get_timezones('Europe/Paris'):
+            with self.subTest(repr(tz)):
+                with timezone.override(tz):
+                    # This is a bug.
+                    self.assertTrue(form.is_valid())
+                    self.assertEqual(form.cleaned_data['dt'], datetime.datetime(2011, 3, 27, 2, 30, 0))
 
     def test_form_with_ambiguous_time(self):
         form = EventForm({'dt': '2011-10-30 02:30:00'})
-        with timezone.override(pytz.timezone('Europe/Paris')):
-            # This is a bug.
-            self.assertTrue(form.is_valid())
-            self.assertEqual(form.cleaned_data['dt'], datetime.datetime(2011, 10, 30, 2, 30, 0))
+        for tz in get_timezones('Europe/Paris'):
+            with self.subTest(repr(tz)):
+                with timezone.override(tz):
+                    # This is a bug.
+                    self.assertTrue(form.is_valid())
+                    self.assertEqual(form.cleaned_data['dt'], datetime.datetime(2011, 10, 30, 2, 30, 0))
 
     def test_split_form(self):
         form = EventSplitForm({'dt_0': '2011-09-01', 'dt_1': '13:20:30'})
@@ -1082,8 +1145,15 @@ class LegacyFormsTests(TestCase):
         self.assertEqual(e.dt, datetime.datetime(2011, 9, 1, 13, 20, 30))
 
 
+# RemovedInDjango50Warning: When the deprecation ends, remove setUpClass() and
+# USE_L10N=False. The tests should remain because format-related settings will
+# take precedence over locale-dictated formats.
 @override_settings(DATETIME_FORMAT='c', TIME_ZONE='Africa/Nairobi', USE_L10N=False, USE_TZ=True)
 class NewFormsTests(TestCase):
+    @classmethod
+    def setUpClass(cls):
+        with ignore_warnings(category=RemovedInDjango50Warning):
+            super().setUpClass()
 
     @requires_tz_support
     def test_form(self):
@@ -1098,26 +1168,30 @@ class NewFormsTests(TestCase):
             self.assertEqual(form.cleaned_data['dt'], datetime.datetime(2011, 9, 1, 10, 20, 30, tzinfo=UTC))
 
     def test_form_with_non_existent_time(self):
-        with timezone.override(pytz.timezone('Europe/Paris')):
-            form = EventForm({'dt': '2011-03-27 02:30:00'})
-            self.assertFalse(form.is_valid())
-            self.assertEqual(
-                form.errors['dt'], [
-                    '2011-03-27 02:30:00 couldn’t be interpreted in time zone '
-                    'Europe/Paris; it may be ambiguous or it may not exist.'
-                ]
-            )
+        for tz in get_timezones('Europe/Paris'):
+            with self.subTest(repr(tz)):
+                with timezone.override(tz):
+                    form = EventForm({'dt': '2011-03-27 02:30:00'})
+                    self.assertFalse(form.is_valid())
+                    self.assertEqual(
+                        form.errors['dt'], [
+                            '2011-03-27 02:30:00 couldn’t be interpreted in time zone '
+                            'Europe/Paris; it may be ambiguous or it may not exist.'
+                        ]
+                    )
 
     def test_form_with_ambiguous_time(self):
-        with timezone.override(pytz.timezone('Europe/Paris')):
-            form = EventForm({'dt': '2011-10-30 02:30:00'})
-            self.assertFalse(form.is_valid())
-            self.assertEqual(
-                form.errors['dt'], [
-                    '2011-10-30 02:30:00 couldn’t be interpreted in time zone '
-                    'Europe/Paris; it may be ambiguous or it may not exist.'
-                ]
-            )
+        for tz in get_timezones('Europe/Paris'):
+            with self.subTest(repr(tz)):
+                with timezone.override(tz):
+                    form = EventForm({'dt': '2011-10-30 02:30:00'})
+                    self.assertFalse(form.is_valid())
+                    self.assertEqual(
+                        form.errors['dt'], [
+                            '2011-10-30 02:30:00 couldn’t be interpreted in time zone '
+                            'Europe/Paris; it may be ambiguous or it may not exist.'
+                        ]
+                    )
 
     @requires_tz_support
     def test_split_form(self):
@@ -1152,6 +1226,10 @@ class NewFormsTests(TestCase):
     ROOT_URLCONF='timezones.urls',
 )
 class AdminTests(TestCase):
+    @classmethod
+    def setUpClass(cls):
+        with ignore_warnings(category=RemovedInDjango50Warning):
+            super().setUpClass()
 
     @classmethod
     def setUpTestData(cls):
