@@ -1,12 +1,11 @@
 import string
 import uuid
 
-from django.conf.urls import url as conf_url
 from django.core.exceptions import ImproperlyConfigured
 from django.test import SimpleTestCase
 from django.test.utils import override_settings
 from django.urls import NoReverseMatch, Resolver404, path, resolve, reverse
-from django.utils.deprecation import RemovedInDjango40Warning
+from django.views import View
 
 from .converters import DynamicConverter
 from .views import empty_view
@@ -143,6 +142,19 @@ class SimplifiedURLTests(SimpleTestCase):
         with self.assertRaisesMessage(ImproperlyConfigured, msg):
             path('foo/<nonexistent:var>/', empty_view)
 
+    def test_invalid_view(self):
+        msg = 'view must be a callable or a list/tuple in the case of include().'
+        with self.assertRaisesMessage(TypeError, msg):
+            path('articles/', 'invalid_view')
+
+    def test_invalid_view_instance(self):
+        class EmptyCBV(View):
+            pass
+
+        msg = 'view must be a callable, pass EmptyCBV.as_view(), not EmptyCBV().'
+        with self.assertRaisesMessage(TypeError, msg):
+            path('foo', EmptyCBV())
+
     def test_whitespace_in_route(self):
         msg = (
             "URL route 'space/<int:num>/extra/<str:%stest>' cannot contain "
@@ -157,6 +169,19 @@ class SimplifiedURLTests(SimpleTestCase):
         match = p.resolve('space%s/1/' % string.whitespace)
         self.assertEqual(match.kwargs, {'num': 1})
 
+    def test_path_trailing_newlines(self):
+        tests = [
+            '/articles/2003/\n',
+            '/articles/2010/\n',
+            '/en/foo/\n',
+            '/included_urls/extra/\n',
+            '/regex/1/\n',
+            '/users/1/\n',
+        ]
+        for url in tests:
+            with self.subTest(url=url), self.assertRaises(Resolver404):
+                resolve(url)
+
 
 @override_settings(ROOT_URLCONF='urlpatterns.converter_urls')
 class ConverterTests(SimpleTestCase):
@@ -315,13 +340,3 @@ class ConversionExceptionTests(SimpleTestCase):
             raise TypeError('This type error propagates.')
         with self.assertRaisesMessage(TypeError, 'This type error propagates.'):
             reverse('dynamic', kwargs={'value': object()})
-
-
-class DeprecationTests(SimpleTestCase):
-    def test_url_warning(self):
-        msg = (
-            'django.conf.urls.url() is deprecated in favor of '
-            'django.urls.re_path().'
-        )
-        with self.assertRaisesMessage(RemovedInDjango40Warning, msg):
-            conf_url(r'^regex/(?P<pk>[0-9]+)/$', empty_view, name='regex')
