@@ -2,6 +2,7 @@ from datetime import datetime
 
 from django.core.exceptions import ValidationError
 from django.db import models
+from django.db.models.functions import Lower
 
 
 def validate_answer_to_universe(value):
@@ -74,8 +75,17 @@ class CustomMessagesModel(models.Model):
     )
 
 
+class AuthorManager(models.Manager):
+    def get_queryset(self):
+        qs = super().get_queryset()
+        return qs.filter(archived=False)
+
+
 class Author(models.Model):
     name = models.CharField(max_length=100)
+    archived = models.BooleanField(default=False)
+
+    objects = AuthorManager()
 
 
 class Article(models.Model):
@@ -94,9 +104,6 @@ class Post(models.Model):
     subtitle = models.CharField(max_length=50, unique_for_month='posted', blank=True)
     posted = models.DateField()
 
-    def __str__(self):
-        return self.name
-
 
 class FlexibleDatePost(models.Model):
     title = models.CharField(max_length=50, unique_for_date='posted', blank=True)
@@ -121,16 +128,11 @@ class GenericIPAddrUnpackUniqueTest(models.Model):
     generic_v4unpack_ip = models.GenericIPAddressField(null=True, blank=True, unique=True, unpack_ipv4=True)
 
 
-# A model can't have multiple AutoFields
-# Refs #12467.
-assertion_error = None
-try:
-    class MultipleAutoFields(models.Model):
-        auto1 = models.AutoField(primary_key=True)
-        auto2 = models.AutoField(primary_key=True)
-except AssertionError as exc:
-    assertion_error = exc
-assert str(assertion_error) == (
-    "Model validation.MultipleAutoFields can't have more than one "
-    "auto-generated field."
-)
+class UniqueFuncConstraintModel(models.Model):
+    field = models.CharField(max_length=255)
+
+    class Meta:
+        required_db_features = {'supports_expression_indexes'}
+        constraints = [
+            models.UniqueConstraint(Lower('field'), name='func_lower_field_uq'),
+        ]
