@@ -12,7 +12,7 @@ from unittest import mock
 from django.core import mail
 from django.core.files.uploadedfile import SimpleUploadedFile
 from django.db import DatabaseError, connection
-from django.http import Http404
+from django.http import Http404, HttpRequest, HttpResponse
 from django.shortcuts import render
 from django.template import TemplateDoesNotExist
 from django.test import RequestFactory, SimpleTestCase, override_settings
@@ -23,8 +23,8 @@ from django.utils.functional import SimpleLazyObject
 from django.utils.regex_helper import _lazy_re_compile
 from django.utils.safestring import mark_safe
 from django.views.debug import (
-    CallableSettingWrapper, ExceptionReporter, Path as DebugPath,
-    SafeExceptionReporterFilter, default_urlconf,
+    CallableSettingWrapper, ExceptionCycleWarning, ExceptionReporter,
+    Path as DebugPath, SafeExceptionReporterFilter, default_urlconf,
     get_default_exception_reporter_filter, technical_404_response,
     technical_500_response,
 )
@@ -86,6 +86,16 @@ class DebugViewTests(SimpleTestCase):
             response = self.client.get('/raises400/')
         self.assertContains(response, '<div class="context" id="', status_code=400)
 
+    def test_400_bad_request(self):
+        # When DEBUG=True, technical_500_template() is called.
+        with self.assertLogs('django.request', 'WARNING') as cm:
+            response = self.client.get('/raises400_bad_request/')
+        self.assertContains(response, '<div class="context" id="', status_code=400)
+        self.assertEqual(
+            cm.records[0].getMessage(),
+            'Malformed request syntax: /raises400_bad_request/',
+        )
+
     # Ensure no 403.html template exists to test the default case.
     @override_settings(TEMPLATES=[{
         'BACKEND': 'django.template.backends.django.DjangoTemplates',
@@ -112,14 +122,35 @@ class DebugViewTests(SimpleTestCase):
 
     def test_404(self):
         response = self.client.get('/raises404/')
-        self.assertEqual(response.status_code, 404)
-        self.assertContains(response, "<code>not-in-urls</code>, didn't match", status_code=404)
+        self.assertNotContains(
+            response,
+            '<pre class="exception_value">',
+            status_code=404,
+        )
+        self.assertContains(
+            response,
+            '<p>The current path, <code>not-in-urls</code>, didn’t match any '
+            'of these.</p>',
+            status_code=404,
+            html=True,
+        )
 
     def test_404_not_in_urls(self):
         response = self.client.get('/not-in-urls')
         self.assertNotContains(response, "Raised by:", status_code=404)
+        self.assertNotContains(
+            response,
+            '<pre class="exception_value">',
+            status_code=404,
+        )
         self.assertContains(response, "Django tried these URL patterns", status_code=404)
-        self.assertContains(response, "<code>not-in-urls</code>, didn't match", status_code=404)
+        self.assertContains(
+            response,
+            '<p>The current path, <code>not-in-urls</code>, didn’t match any '
+            'of these.</p>',
+            status_code=404,
+            html=True,
+        )
         # Pattern and view name of a RegexURLPattern appear.
         self.assertContains(response, r"^regex-post/(?P&lt;pk&gt;[0-9]+)/$", status_code=404)
         self.assertContains(response, "[name='regex-post']", status_code=404)
@@ -130,23 +161,49 @@ class DebugViewTests(SimpleTestCase):
     @override_settings(ROOT_URLCONF=WithoutEmptyPathUrls)
     def test_404_empty_path_not_in_urls(self):
         response = self.client.get('/')
-        self.assertContains(response, "The empty path didn't match any of these.", status_code=404)
+        self.assertContains(
+            response,
+            '<p>The empty path didn’t match any of these.</p>',
+            status_code=404,
+            html=True,
+        )
 
     def test_technical_404(self):
         response = self.client.get('/technical404/')
+        self.assertContains(
+            response,
+            '<pre class="exception_value">Testing technical 404.</pre>',
+            status_code=404,
+            html=True,
+        )
         self.assertContains(response, "Raised by:", status_code=404)
-        self.assertContains(response, "view_tests.views.technical404", status_code=404)
+        self.assertContains(
+            response,
+            '<td>view_tests.views.technical404</td>',
+            status_code=404,
+        )
+        self.assertContains(
+            response,
+            '<p>The current path, <code>technical404/</code>, matched the '
+            'last one.</p>',
+            status_code=404,
+            html=True,
+        )
 
     def test_classbased_technical_404(self):
         response = self.client.get('/classbased404/')
-        self.assertContains(response, "Raised by:", status_code=404)
-        self.assertContains(response, "view_tests.views.Http404View", status_code=404)
+        self.assertContains(
+            response,
+            '<th>Raised by:</th><td>view_tests.views.Http404View</td>',
+            status_code=404,
+            html=True,
+        )
 
     def test_non_l10ned_numeric_ids(self):
         """
         Numeric IDs and fancy traceback context blocks line numbers shouldn't be localized.
         """
-        with self.settings(DEBUG=True, USE_L10N=True):
+        with self.settings(DEBUG=True):
             with self.assertLogs('django.request', 'ERROR'):
                 response = self.client.get('/raises500/')
             # We look for a HTML fragment of the form
@@ -204,14 +261,13 @@ class DebugViewTests(SimpleTestCase):
     @override_settings(ROOT_URLCONF='view_tests.default_urls')
     def test_default_urlconf_template(self):
         """
-        Make sure that the default URLconf template is shown shown instead
-        of the technical 404 page, if the user has not altered their
-        URLconf yet.
+        Make sure that the default URLconf template is shown instead of the
+        technical 404 page, if the user has not altered their URLconf yet.
         """
         response = self.client.get('/')
         self.assertContains(
             response,
-            "<h2>The install worked successfully! Congratulations!</h2>"
+            "<h1>The install worked successfully! Congratulations!</h1>"
         )
 
     @override_settings(ROOT_URLCONF='view_tests.regression_21530_urls')
@@ -260,6 +316,21 @@ class DebugViewTests(SimpleTestCase):
             response = self.client.get('/raises500/')
         self.assertContains(response, 'custom traceback text', status_code=500)
 
+    @override_settings(DEFAULT_EXCEPTION_REPORTER='view_tests.views.TemplateOverrideExceptionReporter')
+    def test_template_override_exception_reporter(self):
+        with self.assertLogs('django.request', 'ERROR'):
+            response = self.client.get('/raises500/')
+        self.assertContains(
+            response,
+            '<h1>Oh no, an error occurred!</h1>',
+            status_code=500,
+            html=True,
+        )
+
+        with self.assertLogs('django.request', 'ERROR'):
+            response = self.client.get('/raises500/', HTTP_ACCEPT='text/plain')
+        self.assertContains(response, 'Oh dear, an error occurred!', status_code=500)
+
 
 class DebugViewQueriesAllowedTests(SimpleTestCase):
     # May need a query to initialize MySQL connection
@@ -297,6 +368,16 @@ class NonDjangoTemplatesDebugViewTests(SimpleTestCase):
             response = self.client.get('/raises400/')
         self.assertContains(response, '<div class="context" id="', status_code=400)
 
+    def test_400_bad_request(self):
+        # When DEBUG=True, technical_500_template() is called.
+        with self.assertLogs('django.request', 'WARNING') as cm:
+            response = self.client.get('/raises400_bad_request/')
+        self.assertContains(response, '<div class="context" id="', status_code=400)
+        self.assertEqual(
+            cm.records[0].getMessage(),
+            'Malformed request syntax: /raises400_bad_request/',
+        )
+
     def test_403(self):
         response = self.client.get('/raises403/')
         self.assertContains(response, '<h1>403 Forbidden</h1>', status_code=403)
@@ -358,6 +439,19 @@ class ExceptionReporterTests(SimpleTestCase):
         self.assertIn('<h2>Request information</h2>', html)
         self.assertIn('<p>Request data not supplied</p>', html)
 
+    def test_sharing_traceback(self):
+        try:
+            raise ValueError('Oops')
+        except ValueError:
+            exc_type, exc_value, tb = sys.exc_info()
+        reporter = ExceptionReporter(None, exc_type, exc_value, tb)
+        html = reporter.get_traceback_html()
+        self.assertIn(
+            '<form action="https://dpaste.com/" name="pasteform" '
+            'id="pasteform" method="post">',
+            html,
+        )
+
     def test_eol_support(self):
         """The ExceptionReporter supports Unix, Windows and Macintosh EOL markers"""
         LINES = ['print %d' % i for i in range(1, 6)]
@@ -391,6 +485,100 @@ class ExceptionReporterTests(SimpleTestCase):
         self.assertIn('<h2>Request information</h2>', html)
         self.assertNotIn('<p>Request data not supplied</p>', html)
 
+    def test_suppressed_context(self):
+        try:
+            try:
+                raise RuntimeError("Can't find my keys")
+            except RuntimeError:
+                raise ValueError("Can't find my keys") from None
+        except ValueError:
+            exc_type, exc_value, tb = sys.exc_info()
+
+        reporter = ExceptionReporter(None, exc_type, exc_value, tb)
+        html = reporter.get_traceback_html()
+        self.assertInHTML('<h1>ValueError</h1>', html)
+        self.assertIn('<pre class="exception_value">Can&#x27;t find my keys</pre>', html)
+        self.assertIn('<th>Exception Type:</th>', html)
+        self.assertIn('<th>Exception Value:</th>', html)
+        self.assertIn('<h2>Traceback ', html)
+        self.assertIn('<h2>Request information</h2>', html)
+        self.assertIn('<p>Request data not supplied</p>', html)
+        self.assertNotIn('During handling of the above exception', html)
+
+    def test_innermost_exception_without_traceback(self):
+        try:
+            try:
+                raise RuntimeError('Oops')
+            except Exception as exc:
+                new_exc = RuntimeError('My context')
+                exc.__context__ = new_exc
+                raise
+        except Exception:
+            exc_type, exc_value, tb = sys.exc_info()
+
+        reporter = ExceptionReporter(None, exc_type, exc_value, tb)
+        frames = reporter.get_traceback_frames()
+        self.assertEqual(len(frames), 2)
+        html = reporter.get_traceback_html()
+        self.assertInHTML('<h1>RuntimeError</h1>', html)
+        self.assertIn('<pre class="exception_value">Oops</pre>', html)
+        self.assertIn('<th>Exception Type:</th>', html)
+        self.assertIn('<th>Exception Value:</th>', html)
+        self.assertIn('<h2>Traceback ', html)
+        self.assertIn('<h2>Request information</h2>', html)
+        self.assertIn('<p>Request data not supplied</p>', html)
+        self.assertIn(
+            'During handling of the above exception (My context), another '
+            'exception occurred',
+            html,
+        )
+        self.assertInHTML('<li class="frame user">None</li>', html)
+        self.assertIn('Traceback (most recent call last):\n  None', html)
+
+        text = reporter.get_traceback_text()
+        self.assertIn('Exception Type: RuntimeError', text)
+        self.assertIn('Exception Value: Oops', text)
+        self.assertIn('Traceback (most recent call last):\n  None', text)
+        self.assertIn(
+            'During handling of the above exception (My context), another '
+            'exception occurred',
+            text,
+        )
+
+    def test_mid_stack_exception_without_traceback(self):
+        try:
+            try:
+                raise RuntimeError('Inner Oops')
+            except Exception as exc:
+                new_exc = RuntimeError('My context')
+                new_exc.__context__ = exc
+                raise RuntimeError('Oops') from new_exc
+        except Exception:
+            exc_type, exc_value, tb = sys.exc_info()
+        reporter = ExceptionReporter(None, exc_type, exc_value, tb)
+        html = reporter.get_traceback_html()
+        self.assertInHTML('<h1>RuntimeError</h1>', html)
+        self.assertIn('<pre class="exception_value">Oops</pre>', html)
+        self.assertIn('<th>Exception Type:</th>', html)
+        self.assertIn('<th>Exception Value:</th>', html)
+        self.assertIn('<h2>Traceback ', html)
+        self.assertInHTML('<li class="frame user">Traceback: None</li>', html)
+        self.assertIn(
+            'During handling of the above exception (Inner Oops), another '
+            'exception occurred:\n  Traceback: None',
+            html,
+        )
+
+        text = reporter.get_traceback_text()
+        self.assertIn('Exception Type: RuntimeError', text)
+        self.assertIn('Exception Value: Oops', text)
+        self.assertIn('Traceback (most recent call last):', text)
+        self.assertIn(
+            'During handling of the above exception (Inner Oops), another '
+            'exception occurred:\n  Traceback: None',
+            text,
+        )
+
     def test_reporting_of_nested_exceptions(self):
         request = self.rf.get('/test_view/')
         try:
@@ -518,7 +706,12 @@ class ExceptionReporterTests(SimpleTestCase):
 
         tb_frames = None
         tb_generator = threading.Thread(target=generate_traceback_frames, daemon=True)
-        tb_generator.start()
+        msg = (
+            "Cycle in the exception chain detected: exception 'inner' "
+            "encountered again."
+        )
+        with self.assertWarnsMessage(ExceptionCycleWarning, msg):
+            tb_generator.start()
         tb_generator.join(timeout=5)
         if tb_generator.is_alive():
             # tb_generator is a daemon that runs until the main thread/process
@@ -548,7 +741,7 @@ class ExceptionReporterTests(SimpleTestCase):
         self.assertIn('<th>Request URL:</th>', html)
         self.assertNotIn('<th>Exception Type:</th>', html)
         self.assertNotIn('<th>Exception Value:</th>', html)
-        self.assertNotIn('<h2>Traceback ', html)
+        self.assertIn('<h2>Traceback ', html)
         self.assertIn('<h2>Request information</h2>', html)
         self.assertNotIn('<p>Request data not supplied</p>', html)
 
@@ -561,7 +754,7 @@ class ExceptionReporterTests(SimpleTestCase):
         self.assertNotIn('<th>Request URL:</th>', html)
         self.assertNotIn('<th>Exception Type:</th>', html)
         self.assertNotIn('<th>Exception Value:</th>', html)
-        self.assertNotIn('<h2>Traceback ', html)
+        self.assertIn('<h2>Traceback ', html)
         self.assertIn('<h2>Request information</h2>', html)
         self.assertIn('<p>Request data not supplied</p>', html)
 
@@ -757,6 +950,20 @@ class ExceptionReporterTests(SimpleTestCase):
             reporter.get_traceback_text()
             m.assert_called_once_with(encoding='utf-8')
 
+    @override_settings(ALLOWED_HOSTS=['example.com'])
+    def test_get_raw_insecure_uri(self):
+        factory = RequestFactory(HTTP_HOST='evil.com')
+        tests = [
+            ('////absolute-uri', 'http://evil.com//absolute-uri'),
+            ('/?foo=bar', 'http://evil.com/?foo=bar'),
+            ('/path/with:colons', 'http://evil.com/path/with:colons'),
+        ]
+        for url, expected in tests:
+            with self.subTest(url=url):
+                request = factory.get(url)
+                reporter = ExceptionReporter(request, None, None, None)
+                self.assertEqual(reporter._get_raw_insecure_uri(), expected)
+
 
 class PlainTextReportTests(SimpleTestCase):
     rf = RequestFactory()
@@ -1249,6 +1456,19 @@ class ExceptionReporterFilterTests(ExceptionReportTestMixin, LoggingCaptureMixin
             {'login': 'cooper', 'password': reporter_filter.cleansed_substitute},
         )
 
+    def test_cleanse_setting_recurses_in_dictionary_with_non_string_key(self):
+        reporter_filter = SafeExceptionReporterFilter()
+        initial = {('localhost', 8000): {'login': 'cooper', 'password': 'secret'}}
+        self.assertEqual(
+            reporter_filter.cleanse_setting('SETTING_NAME', initial),
+            {
+                ('localhost', 8000): {
+                    'login': 'cooper',
+                    'password': reporter_filter.cleansed_substitute,
+                },
+            },
+        )
+
     def test_cleanse_setting_recurses_in_list_tuples(self):
         reporter_filter = SafeExceptionReporterFilter()
         initial = [
@@ -1399,7 +1619,7 @@ class NonHTMLResponseExceptionReporterFilter(ExceptionReportTestMixin, LoggingCa
     @override_settings(DEBUG=True, ROOT_URLCONF='view_tests.urls')
     def test_non_html_response_encoding(self):
         response = self.client.get('/raises500/', HTTP_ACCEPT='application/json')
-        self.assertEqual(response['Content-Type'], 'text/plain; charset=utf-8')
+        self.assertEqual(response.headers['Content-Type'], 'text/plain; charset=utf-8')
 
 
 class DecoratorsTests(SimpleTestCase):
@@ -1423,3 +1643,17 @@ class DecoratorsTests(SimpleTestCase):
             @sensitive_post_parameters
             def test_func(request):
                 return index_page(request)
+
+    def test_sensitive_post_parameters_http_request(self):
+        class MyClass:
+            @sensitive_post_parameters()
+            def a_view(self, request):
+                return HttpResponse()
+
+        msg = (
+            "sensitive_post_parameters didn't receive an HttpRequest object. "
+            "If you are decorating a classmethod, make sure to use "
+            "@method_decorator."
+        )
+        with self.assertRaisesMessage(TypeError, msg):
+            MyClass().a_view(HttpRequest())
